proc template;

    /*******************************************************************/
    /*******************************************************************/
    /*******************************************************************/
    /* SASReport.  Shows all events relevant to SAS Report XML         */
    /* Support:    Eric Gebhart (saseag)                               */
    /*******************************************************************/
    /*******************************************************************/
    /*******************************************************************/
     %macro tagsets_sasreport;

     %if  %substr(&sysver,1,1) = 8 %then %do;
         /* not in version 8 */
         %end;

    %else %do;

    define tagset tagsets.SASReport15;
        private;
        parent = tagsets.SASReport13;
    end;

    define tagset tagsets.SASReport14;
        private;
        parent = tagsets.SASReport13;
    end;

    /* S0693849 - Backport SASReport12 for 9.3 */
    define tagset tagsets.SASReport12;
        parent = tagsets.SASReport13;

        define event doc;
         start:
            put '<?xml version="1.0"';
            putq " encoding=" encoding;
            put "?>" CR CR;
            put '<SASReport version="1.2" applicationName="ODS" applicationVersion="9.3"' nl;
            put ' xmlns:style="http://www.sas.com/sasreportmodel/styles"' nl;
            put ' xmlns:ods="http://www.sas.com/sasreportmodel/ods"' nl;
            putq ' dateCreated="'; /* 2009-11-10" */
            put javadate 'T' javatime '"';
            putq ' dateModified="';
            put javadate 'T' javatime '"';
            put ">" nl;
            ndent;
            eval $layout_nesting_level 0;
         finish:
            xdent;
            put $$view;
            unset $$view;
            do / if any($$mappings,$$format);
               put '<Formats>' nl;
               put $$mappings;
               unset $$mappings;
               put $$format;
               unset $$format;
               put '</Formats>' nl;
            done;
            do /if $$Actions;
                put "<Actions>" nl;
                put $$Actions;
                put "</Actions>" nl;
            done;
            put '</SASReport>' nl;
        end; /* doc */
    end; /* end SASReport12 */


    define tagset tagsets.SASReport13;
        parent = tagsets.graph;
        has_layout_support = yes;
        notes "This is the SAS Report XML event map definition";
        /*log_note = 'This tagset is used for data interchange';*/

        /* %let  map =<>%nrstr(&)%str(%')%str(%");
        map="&map"; */
        map = '<>&''"';
        mapsub = '/&lt;/&gt;/&amp;/&#39;/&quot;/';
        /*map = '<>&';
        mapsub = '/&lt;/&gt;/&amp;/';*/
        nobreakspace = '<NBSP/>';
        split = '<Br/>';

    /*---------------------------------------------------------sasdck-*/
    /*-- Indent must be zero.  Otherwise preformatted text messes up.--*/
    /*--------------------------------------------------------28Apr03-*/

        indent=0;
        stacked_columns = yes;
        hierarchical_data = yes;
        output_type = 'xml';
        copyright='&#169;';
        trademark='&#153;';
        registered_tm='&#174;';
        image_formats = 'png,gif,jpg,xml';
        no_byte_order_mark = yes;
        embedded_stylesheet = yes;

        breaktext_ratio = 2.5;
        breaktext_width = 40;
        breaktext_length = 80;

        mvar _ODSM_SR_RAW;

        /*default_event="basic";*/

        /*-------------------------------------------------------eric-*/
        /*-- Handy function for debugging.                          --*/
        /*----------------------------------------------------29Jan03-*/
        define event putvars;
            put "event vars" nl;
            putvars event "name: " _name_ " = " _value_ nl;
            put "memory vars" nl;
            putvars mem "name: " _name_ " = " _value_ nl;
            put "style vars" nl;
            putvars style "name: " _name_ " = " _value_ nl;
        end;

        define event shortstyles;
        end;

        define event basic;
          start:
             put "EVENT: " event_name nl;
          finish:
             put "EVENT/: " event_name nl;
        end;

        define event initialize;
            set $Options['Junk'] 'junk';
            trigger set_options;
            trigger init_border_style;
            trigger set_just_lookup;
            trigger grand_init; /* S0618503 */
            eval $proc_title_count 0;
            eval $link_count 0;
            trigger set_just_lookup;
        end;

        define event options_set;
            trigger set_options;
        end;

        /* this is needed to make spanrows work with proc report. S0464429 */
        /* See also:  S0467067 */
        define event rowspancolspanfill;
            do /if ^$cell_count;
                eval $cell_count 0;
            done;
            do /if $do_csv;
                eval $cell_count $cell_count + 1;
                put ", " /if $cell_count > 1;
                put '""';
            done;
        end;

        define event set_options;
            set $do_csv "True";
            set $do_formatted "True";
            unset $do_csv /if cmp($Options['CSV'], "Off");
            do /if $Options['RAW'];
                unset $do_formatted /if cmp($Options['RAW'], "On");
            else;
                unset $do_formatted /if cmp(_ODSM_SR_RAW, "On");
            done;
            trigger documentation;
        end;

        define event documentation;
            trigger help /breakif cmp($options['DOC'], "Help");
            trigger quick_reference /breakif cmp($options['DOC'], "Quick");
        end;

        define event span;
            start:
               put '<Span ';
               trigger style_name_align;
               put '>' ;
               trigger pre_post_text_value start;
               put value;
               trigger pre_post_text_value finish;
            finish:
               put "</Span>" nl;
        end;

       define event super;
           start:
               put '<Span style:verticalAlign="super">' ;
               put VALUE;
               put "</Span>" ;
       end ;

       define event sub;
           start:
               put '<Span style:verticalAlign="sub">' ;
               put VALUE;
               put "</Span>" ;
        end ;

        define event sigma;
            start:
                put '&#963;';
        end;

        define event dagger;
            start:
                put '&#8224;' ;
        end ;

        define event nbspace;
            do /if value;
                break /if index(value,"-");
                eval $ncount inputn(value, "3.");
                do /if $ncount > 256;
                    eval $ncount 256;
                done;
            else;
                eval $ncount 1;
            done;

            do /while $ncount;
                put '<NBSP/>' ;
                eval $ncount $ncount-1;
            done;
            unset $ncount;
        end;

        define event newline;
            unset $value;
            do /if value;
                break /if index(value,"-");
                eval $ncount inputn(value, "3.");
                do /if $ncount > 256;
                    eval $ncount 256;
                done;
            else;
                eval $ncount 1;
            done;

            do /while $ncount;
                put '<Br/>';
                eval $ncount $ncount-1;
            done;
            unset $ncount;
        end;

        define event unicode ;
         Notes "Unicode function inserts unicode values. ";
         start:
            do /if value;
               set $squote "'" ;
            eval $temp value ;
            set  $temp strip($temp);
            set  $temp upcase($temp);

               /* is it in the internal list ? */
            do /if  $unicodeMap[$temp] ;
               eval $newvalue $unicodeMap[$temp] ;
            else ;
                  /* is it '01B3'x form */
                  do / if (index($temp, $squote ) > 0);
                     set $newvalue scan($temp, 1, $squote );
                  /* is it "01B3"x form */
                  else /if (index($temp, '"') > 0);
                     set $newvalue scan($temp, 1, '"');
                  /* Then it must be 01B3 */
                  else;
               set $newvalue $temp;
                  done;
               done;

            eval $unicode inputn($newvalue, "hex4.") ;
            put '&#' $unicode ';';
            done;
        end;

        define event thispage;
            put "<PageNumber></PageNumber>";
        end;

        define event lastpage;
            put "<PageCount></PageCount>";
        end;

    define event help;
        putlog "==============================================================================";
        putlog "The SASReport Tagset Help Text.";
        putlog " ";
        putlog "This Tagset/Destination creates SAS Report XML";
        putlog "It is used for interoperability between many SAS products.";
        putlog " ";
        putlog "See Also:";
        putlog "http://support.sas.com/rnd/base/topics/odsmarkup/";
        putlog " ";
        trigger quick_reference;
    end;

    define event quick_reference;
        putlog "==============================================================================";
        putlog " ";
        putlog "These are the options supported by this tagset.";
        putlog " ";
        putlog "Sample usage:";
        putlog " ";
        putlog "ods sasreport options(doc='Quick'); ";
        putlog " ";
        putlog "ods sasreport options(csv='Off' Raw='On'); ";
        putlog " ";
        putlog "Doc:  No default value.";
        putlog "     Help: Displays introductory text and options.";
        putlog "     Quick: Displays available options.";
        putlog " ";
        putlog "CSV:   Default Value 'On'";
        do /if $do_csv;
            putlog "     Current Value: On";
        else;
            putlog "     Current Value: Off";
        done;
        putlog "     If 'On' this option causes the embedded data to be in CSV format.";
        putlog "     If Raw is also on, numeric, non integer, values are also given in IEEE,";
        putlog "     Base64 encoded text";
        putlog " ";
        putlog "     Variable types are also specified, instead of using the 'string' type";
        putlog "     for everything.";
        putlog " ";
        putlog "     If 'Off' the embedded data will be in values tags and all data will be";
        putlog "     formatted with a type of 'String'";
        putlog " ";
        putlog "Raw:   Default Value 'Off'";
        do /if $do_formatted;
            putlog "     Current Value: Off";
        else;
            putlog "     Current Value: On";
        done;
        putlog "     This option works in conjunction with the csv option.";
        putlog "     If 'Off' all numeric values will be formatted as text instead of encoded";
        putlog "     in IEEE format.  This can also be turned on by setting the _ODSM_SR_RAW";
        putlog "     macro variable to 'On'.  Options always win over the macro variable.";
        putlog " ";
        putlog "==============================================================================";
    end;

        /* get rid of event defined by graph */
        define event javascript;
        end;

        /* Same as put_value for now. Ignore url variable. */
        /* obsolete event - dch
        define event hyperlink;
            trigger put_value;
        end; */

        /* Put value in Data */
        define event put_value;
            do /if $in_csv;
                do /if $data_note;
                    trigger table_label start;
                    put value;
                else;
                    do /if $do_formatted;
                        set $string value;
                        trigger clean_string;
                        put tranwrd($string, '<Br/>', ' ');
                        /*put tranwrd($value, '"', '""');*/
                        unset $value;
                        unset $string;
                    else;
                        put rawvalue;
                    done;
                done;
            else;
                do /if $data_note;
                    trigger table_label start;
                    put value;
                else;
                    /*-------------------------------------------eric-*/
                    /*-- Graph is inserting an image map rendered   --*/
                    /*-- by java, that string should not            --*/
                    /*-- be cleaned.                                --*/
                    /*----------------------------------------3Aug 07-*/
                    set $string value;
                    trigger clean_string /if !$image_map;
                    put $string;
                    unset $string;
                done;
            done;
        end;

        /* Put value in Data */
        define event put_value_cr;
            do /if $in_csv & ^$data_note;
                do /if $do_formatted;
                    set $value tranwrd(value, '<Br/>', ' ');
                    put tranwrd($value, '"', '""') nl;
                    unset $value;
                else;
                    put rawvalue;
                done;
            else;
                put value nl;
            done;
        end;

        /* Put value in View */
        define event put_label;
            put value;
        end;

        define event parse_size_unit;
            eval $size_unit_re prxparse('(([0-9]*[\.]?[0-9]*)(pt|px|in|em|ex|cm|mm|pc)?)') ;
            eval $match prxmatch($size_unit_re, $size_unit);
            set $size_unit_size prxposn($size_unit_re, 1, $size_unit) ;
            set $size_unit_unit prxposn($size_unit_re, 2, $size_unit) ;
            set $size_unit_unit lowcase($size_unit_unit) /if $size_unit_unit;
            set $size_unit_unit "px" /if !$size_unit_unit;
            unset $match;
        end;

        define event paragraph;
            do /if ^$ODSTEXT;
                eval $ODSTEXT 1;
            else;
                eval $ODSTEXT $ODSTEXT+1;
            done;

            open view;
                do /if $in_page_headers;
                    put '</Header>' nl;
                    unset $in_page_headers;
                done;
                trigger procnoteheaders finish;
                trigger proctitleheaders finish;
                do /if ^$in_container;
                     put "<Section" " name=" '"ODSTEXT_' $ODSTEXT '">' nl;
                     put "<Body>" nl;
                done;
                trigger body_text start;
                trigger body_text finish;
            open view;
                do /if ^$in_container;
                   put "</Body>" nl;
                   put "</Section>" nl;
                done;
            close;
        end;

        define event embedded_stylesheet;
            start:
                put "<style:CSSScheme" ;
                putq " name=" style;
                put ">" nl;
            finish:
                put "</style:CSSScheme>" nl;
        end;

        define event style_name_no_align;
           trigger stylename;
           trigger page_break_before;
           trigger style_inline;
       end;

        define event style_name_align;
           trigger stylename;
           trigger page_break_before;
           trigger text_align;
           trigger style_inline;
        end;

        /* I'm betting this comes back to life at somepoint. */
        define event stylename;
            break /if $prepost;
            putq ' style:style=' htmlclass ;
            /*putq ' Asis=' asis ;*/
        end;

        define event preformatted;
            put ' style:whiteSpace="pre"' /if asis;
        end;

        define event page_break_before;
           unset $breakpage;
        end;

        define event text_align;
            break /if ^just;
            put  ' style:textAlign=';
            put  '"center"' / if cmp(just, 'c');
            put  '"left"'   / if cmp(just, 'l');
            put  '"right"'  / if cmp(just, 'r');
            put  '"right"'  / if cmp(just, 'd');
        end;

        define event vert_text_align;
            put  ' style:verticalAlign=';
            do /if ^vjust;
                put  '"middle"' ;
            else;
                put  '"middle"' / if cmp(vjust, 'c');
                put  '"middle"' / if cmp(vjust, 'm');
                put  '"bottom"' / if cmp(vjust, 'b');
                put  '"top"'    / if cmp(vjust, 't');
            done;
        end;

        /*-------------------------------------------------------eric-*/
        /*-- Loop through the style variables creating attributes   --*/
        /*-- that look like this.                                   --*/
        /*--                                                        --*/
        /*-- style:attribute="value"                                --*/
        /*----------------------------------------------------30Apr03-*/
        define event style_inline;
            putq " style:fontFamily=" font_face;
            putq " style:fontSize=" font_size;
            putq " style:fontWeight=" font_weight;
            putq " style:fontStyle=" font_style;

            putq " style:color=" foreground;
            putq " style:backgroundColor=" background;
            putq " style:backgroundImage=" backgroundimage;

            putq " style:marginLeft=" leftmargin;
            putq " style:marginRight=" rightmargin;
            putq " style:marginTop=" topmargin;
            putq " style:marginBottom=" bottommargin;

            putq " style:listStyleType=" bullet;

            putq " style:height=" outputheight;
            putq " style:width=" outputwidth;

            putq " style:textIndent=" indent;
            putq " style:textDecoration=" text_decoration;

            do /if !borderwidth;
                do /if any(bordercolor, borderstyle);
                    put ' style:borderWidth="1px"';
                done;
            else;
                putq " style:borderWidth=" borderwidth;
            done;
            do / if !cmp(borderwidth, bordertopwidth, borderbottomwidth,
                         borderrightwidth, borderleftwidth);
                putq " style:borderTopWidth=" bordertopwidth;
                putq " style:borderBottomWidth=" borderbottomwidth;
                putq " style:borderRightWidth=" borderrightwidth;
                putq " style:borderLeftWidth=" borderleftwidth;
            done;

            do /if !bordercolor;
                do /if any(borderwidth, borderstyle);
                    put ' style:borderColor="#000000"';
                done;
            else;
                putq " style:borderColor=" bordercolor;
            done;
            do / if !cmp(bordercolor, bordertopcolor, borderbottomcolor,
                         borderrightcolor, borderleftcolor);
                putq " style:borderTopColor=" bordertopcolor;
                putq " style:borderBottomColor=" borderbottomcolor;
                putq " style:borderRightColor=" borderrightcolor;
                putq " style:borderLeftColor=" borderleftcolor;
            done;

            do /if !borderstyle;
                do /if any(borderwidth, bordercolor);
                    put ' style:borderStyle="solid"';
                done;
            else;
                putq " style:borderStyle=" borderstyle;
            done;
            do / if !cmp(borderstyle, bordertopstyle, borderbottomstyle,
                         borderrightstyle, borderleftstyle);
                putq " style:borderTopStyle=" bordertopstyle;
                putq " style:borderBottomStyle=" borderbottomstyle;
                putq " style:borderRightStyle=" borderrightstyle;
                putq " style:borderLeftStyle=" borderleftstyle;
            done;

            putq " style:whiteSpace=" whitespace;
        end;

        /* Eliminate the Graph definitions that collide */
        /* with the SASReport events to prevent duplication (cdh). */
        define event GraphSASReport;
        end;
        define event GraphData;
        end;
        define event GraphSection;
        end;
        define event GraphBody;
    end;

    /*-----------------------------------------------------------eric-*/
    /*-- This is the event needed to put borders on graphs.  It's   --*/
    /*-- just a matter of figuring out how to specify that on the   --*/
    /*-- graph or img element with out clashes.                     --*/
    /*--                                                            --*/
    /*-- Unfortunately sasreport tagset seems to cause a different  --*/
    /*-- code path.  That code path doesn't do borders...           --*/
    /*--------------------------------------------------------4Nov 04-*/
        define event graph_border;
            start:
                open graph_border;
                putq " style:borderWidth=" BORDERWIDTH;
                putq " style:borderStyle=" BORDERSTYLE;
                put  ' style:borderStyle="solid"' /if ^BORDERSTYLE;
                putq " style:borderColor=" BORDERCOLOR;
                close;
            finish:
                unset $$graph_border;
        end;
    /*-----------------------------------------------------------Dan-*/
    /*-- I need a version of this border output that allows --*/
    /*-- allows me to close the style env by specifying OA_NO_END   --*/
    /*--------------------------------------------------------29Nov 06-*/
        define event live_graph_border;
                open graph_border;
                putq " style:borderWidth=" BORDERWIDTH;
                putq " style:borderStyle=" BORDERSTYLE;
                put  ' style:borderStyle="solid"' /if ^BORDERSTYLE;
                putq " style:borderColor=" BORDERCOLOR;
                close;
        end;

        /* For now, we have to duplicate the <Graph> event to get the */
        /* scheme name assigned correctly.  When tables start using */
        /* styles, this event should be removed and the scheme name */
        /* hard-coded on the <View> element (cdh).                  */
        define event graph;
          start:
            ndent;
            put "<Graph";
            trigger alt_output_name;
            putq " style:height=" @Height / if !cmp(@Height,"0px");
            trigger graph_styleheight / if cmp(@Height,"0px");
            putq " style:width=" @Width / if !cmp(@Width,"0px");
            trigger graph_stylewidth / if cmp(@Width,"0px");
            put ' style:scheme="GraphStyleScheme"';
            put ' style:style="GraphScheme';
            put anchor '" ';
            putq " showChartTips=" @ChartTips;
            putq " view2D=" @View2D;
            do / if $$graph_border;
                put $$graph_border;
                unset $$graph_border;
            done;
            putq " g3dTilt=" @G3DTilt;
            putq " g3dRotate=" @G3DRotate;
            putq " chartBorder=" @ChartBorder;
            putq " drillDownEnabled=" @DrillDownEnabled;
            putq " drillDownMode=" @DrillDownMode;
            putq " rotationX=" @RotateX;
            putq " rotationY=" @RotateY;
            putq " rotationZ=" @RotateZ;
            put ">" CR;
          finish:
            put "</Graph>" CR;
            xdent;
        end;

        define event GraphView;
          start:
             open view;
          finish:
             close;
        end;

        define event StyleScheme;
          start:
            open scheme;
            break / if exists($GraphScheme);
            ndent;
            set $GraphScheme "TRUE";
            put '<style:Scheme name="GraphStyleScheme">' CR;
          finish:
            close;
        end;

        define event fmt;
          start:
            open format;
          finish:
            flush;
            close;
        end;

        define event SchemeOut;
           open scheme;
           put "</style:Scheme>" CR;
           xdent;
           flush;
           close;
           put $$scheme;
           unset $$scheme;
           unset $GraphScheme;
        end;

        define event doc;
         start:
            put '<?xml version="1.0"';
            putq " encoding=" encoding;
            put "?>" CR CR;
            put '<SASReport version="1.3" applicationName="ODS" applicationVersion="9.3"' nl;
            put ' xmlns:style="http://www.sas.com/sasreportmodel/styles"' nl;
            put ' xmlns:ods="http://www.sas.com/sasreportmodel/ods"' nl;
            putq ' dateCreated="'; /* 2009-11-10" */
            put javadate 'T' javatime '"';
            putq ' dateModified="';
            put javadate 'T' javatime '"';
            put ">" nl;
            ndent;
            eval $layout_nesting_level 0;
         finish:
            xdent;
            put $$view;
            unset $$view;
            do / if any($$mappings,$$format);
               put '<Formats>' nl;
               put $$mappings;
               unset $$mappings;
               put $$format;
               unset $$format;
               put '</Formats>' nl;
            done;
            do /if $$Actions;
                put "<Actions>" nl;
                put $$Actions;
                put "</Actions>" nl;
            done;
            put '</SASReport>' nl;
        end; /* doc */

     define event stylesheet_link;
         break /if !exists(url);
         put "<style:CSSScheme";
         putq " name=" style;
         putq " file=" url;
         put "/>" nl;
     end;

     define event break_margin;
         unset $margin_unit;
         eval $match prxmatch($margin_re, $margin);
         set $margin_size prxposn($margin_re, 1, $margin) ;
         /* regex gives back strings... We don't want 0 margins              */
         /* might as well break too, (bif) since we don't want anything else */
         unset $margin_size /breakif inputn($margin_size, "2.3") = 0;
         set $margin_unit prxposn($margin_re, 2, $margin) ;
         set $margin_unit lowcase($margin_unit) /if $margin_unit;
         set $margin_unit "in" /if !$margin_unit;
     end;

     define event Margins;
         eval $margin_re prxparse('(([0-9]*[\.]?[0-9]*)(IN|CM)?)') ;

         set $margin getoption('leftmargin');
         trigger break_margin;
         put ' marginLeft="' $margin_size $margin_unit '"' /if $margin_size;

         set $margin getoption('rightmargin');
         trigger break_margin;
         put ' marginRight="' $margin_size $margin_unit '"' /if $margin_size;

         set $margin getoption('topmargin');
         trigger break_margin;
         put ' marginTop="' $margin_size $margin_unit '"' /if $margin_size;

         set $margin getoption('bottommargin');
         trigger break_margin;
         put ' marginBottom="' $margin_size $margin_unit '"' /if $margin_size;
         unset $margin;
         unset $margin_size;
         unset $margin_unit;
     end;

     define event paperHeightWidth;

         /* tranwrd just makes the regex easier. Get rid of optional quotes */
         set $papersize tranwrd($papersize, '"', " ");
         set $papersize tranwrd($papersize, "'", " ");

         /* could be centimeters, could be quoted, or not...
            default is supposedly inches but could be installation
            dependent.
            (8in 11in);
            ('8in', '11in');
            ("8in", "11in");
            ("8", "11");
         */
         eval $re prxparse('( *([0-9]+) *(IN|CM)* *[,]+ *([0-9]+) *(IN|CM)*.*)') ;
         eval $match prxmatch($re, $papersize);

         set $pwidth prxposn($re, 1, $papersize) ;
         set $pwidth_unit prxposn($re, 2, $papersize) ;
         set $pwidth_unit lowcase($pwidth_unit) /if $pwidth_unit;
         set $pwidth_unit "in" /if !$pwidth_unit;

         set $pheight prxposn($re, 3, $papersize) ;
         set $pheight_unit prxposn($re, 4, $papersize) ;
         set $pheight_unit lowcase($pheight_unit) /if $pheight_unit;
         set $pheight_unit "in" /if !$pheight_unit;

         /* Only if they are non-zero */
         put ' height="' $pheight $pheight_unit '"' /if $pheight;
         put ' width="' $pwidth $pwidth_unit '"' /if $pwidth;

         unset $papersize;
         unset $re;
         unset $pwidth;
         unset $pwidth_unit;
         unset $pheight;
         unset $pheight_unit;

     end;

      define event doc_body;
          start:
              put '<Data refreshPolicy="embedded">' nl;
              open view;
              put '<View' ;
              putq ' style:scheme=' style;
              putq ' style:style=' htmlclass;
              put '>' nl;

              trigger pagesetup;

              /* put in an outermost section for Nikolaj    */
              /* This has the html redirect on it so that   */
              /* the XML will never actually be used by WRS */
              trigger section;

              do /if any(preimage, pretext);
                put "<Section>" nl;
                put "<Body>" nl;
                trigger pre_post_image start;
                trigger pre_post_textheader start;
                put "</Body>" nl;
                put "</Section>" nl;
              else /if any(postimage);
                  set $postimage postimage;
              done;

              unset $pageno;
              unset $orientation;
              unset $papersize;

              close;
          finish:
              put '</Data>' nl;
              open view;
              trigger pagebreak_section finish /if $pagebreak_section;
              /* End of Neko's outer section */
              do /if any(preimage, pretext);
                put "<Section>" nl;
                put "<Body>" nl;
                trigger pre_post_image finish;
                trigger pre_post_textheader finish;
                put "</Body>" nl;
                trigger proctitlesection finish;
                put "</Section>" nl;
              done;
              trigger section;
              put "</View>" nl;
              flush;
              close;
              trigger SchemeOut / if exists($GraphScheme);
      end;

      define event pagesetup;

          put "<PageSetup";


          set $pageno getoption('PAGENO');

          do /if !cmp(event_name, "doc_body");
              stop /if cmp($last_pageno, $pageno);

              put ' startingNumber="' $pageno '"' / if cmp(getoption('NUMBER'),'NUMBER');
          else;
              put ' startingNumber="' $pageno '"' / if cmp(getoption('NUMBER'),'NUMBER');
          done;

          set $last_pageno $pageno;


          set $orientation getoption('ORIENTATION');
          put ' orientation="' $orientation '"';

          /*-- handle (width height) --*/

          set $papersize getoption('PAPERSIZE');

          do /if contains($papersize, "(" );
              trigger paperHeightWidth;
          else;
              put ' size="' $papersize '"';
          done;


          /*-- handle margins --*/

          trigger Margins;

          put ">" nl;
          do /if cmp(event_name, "doc_body");
              put "<PageHeader>" nl;
              put '<GriddedLayout/>' nl;
              put '<Text style:textAlign="right">' nl;
              put '<GriddedConstraint/>' nl;
              put '<Span><Date style:textAlign="right"';
              put ' showDate="false" showTime="false"' / if cmp(getoption('DATE'),'NODATE');
              put '>';
              put javadate 'T' javatime;
              put '</Date>   ';
              put '<PageNumber style:textAlign="right" />';
              put '</Span></Text>' nl;
              put "</PageHeader>" nl;
          done;

          put "</PageSetup>" nl;
      end;

      define event cube_label;
         open view;
         put "<Label";
         putq " position=" position;
         do /if cmp(htmlclass, "ContentFolder");
             putq ' style:style="Header"';
             putq ' style:textAlign="center"';
             putq ' style:verticalAlign="middle"';
         else;
             trigger stylename;
             trigger text_align;
             trigger vert_text_align;
             trigger style_inline;
             trigger preformatted;
         done;
         put ">";
         trigger pre_post_text_value start;
         set $string value;
         do /if contains($string, '&amp' ); /* S0713950 */
            trigger partial_clean_string;
         else;
            trigger clean_string;
         done;
         put  $string;
         unset $string;
         trigger pre_post_text_value finish;
         put "</Label>" nl;
         close;
     end;

     /*
     define event rowspancolspanfill;
            do /if $do_csv;
                put ",";
            done;
        end;
        */

     define event splitline;
         do /if $in_csv;
             put " ";
         else;
             put "<Br/>";
         done;
     end;

     define event breakline;
         do /if $in_csv & ^$data_note;
             put " ";
         else;
             put "<Br/>";
         done;
     end;

     define event print_branch_stack;
         do /if $branch_stack;
             eval $branch_count $branch_stack;
         else;
             break;
         done;

         eval $i 1;
         do /while $i <= $branch_count;
             put "," /if $i > 1;
             put $branch_stack[$i];
             set $last_branch $branch_stack[$i];
             eval $i $i + 1;
         done;
     end;

     define event output_name;

         trigger bygroup_slice;


         break /if ^any($output_label, $branch_stack, $cube_label);
         put ' name="' ;

         trigger print_branch_stack;

         do /if ^cmp($last_branch, $output_label);
             put ',' / if exists($branch_stack, $output_label);

             unset $string;
             set $string $output_label;
             trigger clean_string;
             put $string;
             unset $string;
         done;

         do /if $cube_label;
             do / if !cmp($output_label, $cube_label);
                 put ',' / if exists($output_label, $cube_label);

                 unset $string;
                 set $string $cube_label ;
                 trigger clean_string;
                 put $string;
                 unset $string;
             done;
         done;

         put '"';

     end;

     define event alt_output_name;

         trigger bygroup_slice;

/*
         unset $alt;
         set $alt alt /if alt;
         set $alt label /if !$alt;
         set $alt $output_name /if !$alt;
*/

         set $alt ref_id;

         break /if ^any($alt, $branch_stack);
         put ' name="' ;

         trigger print_branch_stack;

         set $string $alt;
         trigger clean_string;
         set $alt $string;
         unset $string;

         do /if !cmp(event_name, "image");
             put ',' / if exists($branch_stack, $alt);
             put $alt;

         else /if !cmp(url, $alt);
             put ',' / if exists($branch_stack, $alt);
             put $alt;
         done;

         put '"';

     end;

     define event bygroup_slice;
         break /if ^$in_bygroup;

         put ' sliceName="' ;

         trigger print_byvars /if $byvars;

         put '"';

     end;

     define event print_byvars;
         iterate $byvars;
         do /while _name_;
             put ";" /if $not_first;
             set $string _name_;
             trigger clean_string;
             put $string "=" ;
             set $string _value_;
             trigger clean_string;
             put $string;
             unset $string;
             set $not_first "True";
             next $byvars;
         done;
         unset $not_first;
     end;


      define event view_cube;
          style=table;
          start:
              open view;
              trigger text finish;
              trigger page_headers finish;
              trigger proctitleheaders finish;
              trigger bodysection;
              trigger pre_post_image start;
              trigger pre_post_textheader start;
              put "<Table";
              putq " style:style=" htmlclass;
              putq " data=" name;
              do /if cellspacing;
                  set $size_unit cellspacing;
                  trigger parse_size_unit;
                  put ' cellSpacing="' $size_unit_size $size_unit_unit '"';
                  unset $size_unit_size;
                  unset $size_unit_unit;
              done;
              do /if cellpadding;
                  set $size_unit cellpadding;
                  trigger parse_size_unit;
                  put ' cellPadding="' $size_unit_size $size_unit_unit '"';
                  unset $size_unit_size;
                  unset $size_unit_unit;
              done;
              putq  ' style:borderStyle=' $borderStyles[frame];
              trigger output_name;
              put ">" nl;
              trigger griddedconstraint;

              put '<ColumnGroup>' nl;
              put '<Label position="top" style:style="Header" style:textAlign="center"/>';
              put '<Columns>' nl;

              put '<Column>' nl;
              put '<Label position="top" style:style="Header" style:textAlign="center"';
              put ' style:verticalAlign="center"/>' nl;
              put '<Cell style:style="Data" style:textAlign="right" style:verticalAlign="bottom" />' nl;

              put '</Column>' nl;

              put '</Columns>' nl;
              put '</ColumnGroup>' nl;

              put '<RowGroup>' nl;
              put '<Label position="top" style:style="RowHeader" style:textAlign="left"/>' nl;
              put '</RowGroup>' nl;

              close;
          finish:
              open view;
              put "</Table>" nl;
              trigger pre_post_textheader finish;
              trigger pre_post_image finish;
              close;
      end;

      define event cube;
         start:
             set $in_cube "True";
             set $cube_label label;
             trigger view_cube;
             put '<IQData type="MDEmbedded"';
             putq ' name=' name;
             put '>' nl;
             ndent;
             put '<EmbeddedData>' nl;
         finish:
             put '</EmbeddedData>' nl;
             xdent;
             put '</IQData>' nl;
             trigger view_cube;
             unset $in_cube;
             unset $cube_label;
      end;

      define event dimensions;
         start:
            ndent;
            put '<Dimensions>' nl;
         finish:
            xdent;
            put '</Dimensions>' nl;
         end;

      define event dimension;
         start:
            ndent;
            put '<Dimension';

            unset $string;
            set $string name;
            trigger clean_string;
            putq ' name=' $string;

            unset $string;
            set $string label;
            trigger clean_string;
            putq " label=" $string;
            unset $string;
            put '>' nl;
         finish:
            xdent;
            put '</Dimension>' nl;
         end;

      define event hierarchy;
         start:
            ndent;
            put '<Hierarchy';
            putq " name=" name;
            putq " label=" label;
            put '>' nl;
         finish:
            xdent;
            put '</Hierarchy>' nl;
         end;

      define event axes;
        start:
          ndent;
          put "<Axes>" CR;
        finish:
          put "</Axes>" CR;
          xdent;
      end;

      define event axis;
         start:
            ndent;
            put '<Axis';
            putq " name=" name;
            putq " type=" type;
            put '>' nl;
         finish:
            xdent;
            put '</Axis>' nl;
         end;

      define event members;
         start:
            ndent;
             put '<Members>' nl;
            unset $justs;
            unset $vjusts;
         finish:
            xdent;
            put '</Members>' nl;
         end;

      define event member;
          ndent;
          put '<Member';

          /* putlog "MEMBER just" ":" just;
             putlog "MEMBER vjust" ":" vjust; */
          do /if cmp(just, 'd');
              set $justs[] 'r';
          else;
              set $justs[] just;
          done;
          do /if vjust;
              set $vjusts[] vjust;
          else;
              set $vjusts[] 'm';
          done;

          unset $string;
          set $string name;
          /* trigger clean_string;*/
          putq ' name=' $string;
          unset $string;

          putq " dimension=" dimension;
          putq " parentMember=" parentmember;

          /*trigger clean_string;*/
          do /if any(pretext, posttext, label);
              put ' label="';
              set $string pretext;
              trigger clean_string;
              put $string;
              unset $string;
              put label;
              set $string posttext;
              trigger clean_string;
              put $string;
              unset $string;
              put '"';
          done;

          set $string level;
          trigger clean_string;
          putq ' level=' $string;
          unset $string;

          do /if ^$do_formatted; /* S0682917 dch*/
              put ' encoding="Base64"';
          else;
             putq " encoding=" dataencoding;
          done;

          putq " rowSpan=" ROWSPAN /if !cmp(ROWSPAN, '1');
          putq " colSpan=" COLSPAN /if !cmp(COLSPAN, '1');
          trigger stylename;
          trigger style_inline;
          trigger text_align;
          trigger vert_text_align;
          trigger print_type;
          trigger print_format;
          do /if url;
              eval $link_count $link_count + 1;
              put ' action="link' $link_count '"';
          done;
          put '/>' nl;
          xdent;
          trigger do_link;
      end;

      define event tuples;
         start:
            ndent;
            put '<Tuples>' nl;
         finish:
            xdent;
            put '</Tuples>' nl;
         end;


      define event tuple;
         start:
            ndent;
            put '<Tuple>' nl;
         finish:
            xdent;
            put '</Tuple>' nl;
         end;

      define event level;
         ndent;
         put '<Level';
         putq " depth=" depth;
         /* label="" is different than no label attribute at all     */
         do /if label ;
             set $string label;
             trigger clean_string;
             putq ' label=' $string;
             unset $string;
         else;
             putq ' label=""';
         done;
         do /if name ;
             set $string name;
             trigger clean_string;
             putq ' name=' $string;
             unset $string;
         done;
      /* putq " SASFormat=" sasformat;                               */
      /* putq " encoding=" dataencoding;                             */
         put '/>' nl;
         xdent;
         end;

      define event cube_values;
          start:
            do /if $do_csv;
                put '<CSV';
            else;
                put '<ValuesList';
            done;
            putq ' valuesCount=' valuescount;
            putq ' columnAxis=' columnaxis;
            putq ' rowAxis=' rowaxis;
            put '>' nl;
            ndent;
         finish:
            xdent;
            do /if $do_csv;
                put '</CSV>' nl;
            else;
                put '</ValuesList>' nl;
            done;
      end;

      define event row;
         start:
            /*-------------------------------------------------------eric-*/
            /*-- Remove this if we want to start outputing table footers.--*/
             /*----------------------------------------------------28Jan03-*/
            trigger table_label / breakif cmp(section,"foot");
            trigger table_body / if cmp(htmlclass,"batch");
            put '<Values>' nl /if ^$do_csv;
            set $RowData "1" / if cmp(htmlclass,"Body");
            eval $cell_count 0;
            ndent;
         finish:
            /*-------------------------------------------------------eric-*/
            /*-- Remove this if we want to start outputing table footers.--*/
            /*----------------------------------------------------28Jan03-*/
            trigger table_label / breakif cmp(section,"foot");
            xdent;
            do /if $do_csv;
                put ';' nl;
            else;
                put '</Values>' nl;
            done;
            unset $RowData;
            trigger table_body / if cmp(htmlclass,"batch");
      end;

      define event header;
         start:
            trigger data;
         finish:
            trigger data;
      end;

      /* PROC PRINT: use blank as missing value in Header-class values */
      define event missing_data;
          break /if ^missing;
          break /if $do_formatted;
          unset $missing;
          do / if !cmp(htmlclass,"Header");
             set $missing strip(value);
             set $missing " " /if ^$missing;
             set $missing " " /if cmp($missing, "<NBSP/>");
             putq ' missing=' $missing;
          else;
             do / if !cmp(proc_name,"Print");
                set $missing strip(value);
                set $missing " " /if ^$missing;
                set $missing " " /if cmp($missing, "<NBSP/>");
                putq ' missing=' $missing;
             else;
                put  ' missing=" "';
             done;
         done;

      end;

      define event cell_is_empty;
         put ' ';
      end;

      /*----------------------------------------------------------eric-*/
      /*-- This event is called by the row.  Data calls span. when   --*/
      /*-- in the foot section.  It is also called directly by print --*/
      /*-------------------------------------------------------15Feb07-*/
      define event table_label;
          start:
              break /if cmp(proc_name, "report");
              set $table_label "true" /if cmp(event_name, 'table_label');
              set $data_note "True";          /*-- for nested events     --*/
              do /if ^$table_label;
                  set $table_label "true";
                  break;
              done;
              unset $table_label;
              put '<Note';
              putq ' event_name=' event_name;
              /*put '<Label';
              put  ' isVisible="false"' / if !exists(value);
              put  ' isStacked="true"'  / if is_stacked;
              put  ' isStacked="false"' / if !is_stacked;
              */
              do / if position;
                  putq ' position=' position;
              else;
                  putq ' position="bottom"';
              done;
              trigger stylename;
              trigger style_inline;
              trigger text_align;
              trigger vert_text_align;
              put  '>';
              do /if cmp(event_name, "table_label");
                  set $string pretext;
                  trigger clean_string;
                  put $string;
                  unset $string;

                  set $string value;
                  trigger clean_string;
                  put  $string;
                  unset $string;

                  set $string posttext;
                  trigger clean_string;
                  put $string;
                  unset $string;
              done;
          finish:
              break /if cmp(proc_name, "report");
              put '</Note>;' nl;
              /*put "</Label>" nl;*/
              unset $data_note;          /*-- for nested events     --*/
      end;

      define event stacked_cell;
          start:
              unset $stacked_values;
      end;

      define event stacked_value;
          start:
              trigger data;
              set $stacked_values "True";
          finish:
              trigger data;
      end;

      define event stacked_value_header;
          start:
              trigger stacked_value;
          finish:
              trigger stacked_value;
      end;

/*
      define event stacked_cell;
          start:
              set $doing_stacked_values "True";
              do /if ^$do_formatted;
                  set $tmp_do_formatted "True";
                  set $do_formatted "True";
              done;
              trigger data;
              unset $stacked_values;
          finish:
              trigger data;
              do /if $tmp_do_formatted;
                  unset $do_formatted;
                  unset $tmp_do_formatted;
              done;
              unset $doing_stacked_values;
      end;


      define event stacked_value;
          start:
              do /if $stacked_values;
                  put "<Br/>";
              else;
                  set $stacked_values "True";
              done;
              trigger put_value;
      end;
*/

     /*-------------------------------------------------------------------*/
      /*-- PROC REPORT LINE statements.                                  --*/
      /*-------------------------------------------------------------------*/
      define event data_note;
          start:
              trigger table_body start;
              set $data_note "True";          /*-- for nested events     --*/
              trigger compute_ba /breakif !cmp(section, "body");
              put '<Note';
              trigger text_align / if ^cmp($RowData,"1");
              trigger stylename; /*/if ^cmp(htmlclass, 'data');*/
              put ' type="String"';
              trigger style_inline ;
              put ' /' / if exists(empty);
              put '>';
              put nl / breakif exists(empty);
                  set $string pretext;
                  trigger clean_string;
                  put $string;
                  unset $string;

                  put value;

                  set $string posttext;
                  trigger clean_string;
                  put $string;
                  unset $string;
          finish:
              unset $data_note;
              trigger compute_ba /breakif !cmp(section, "body");
              put '</Note>' nl /if ^exists(empty);
      end;

      define event compute_ba;
          start:
            open view;
            put  '<Label';
            trigger event_attr;
            put  ' isVisible="false"' / if !exists(value);
            put  ' isStacked="true"'  / if is_stacked;
            put  ' isStacked="false"' / if !is_stacked;
            putq ' position="bottom"' /if cmp(section, "foot");
            putq ' position="top"' /if cmp(section, "head");
            trigger stylename;
            trigger text_align;
            trigger vert_text_align;
            trigger style_inline;
            trigger preformatted;
            put '>';
            trigger pre_post_text_value start;
            put value;
            trigger pre_post_text_value finish;
          finish:
            put '</Label>' nl;
            close;
      end;

      define event do_justs;
          do /if cmp($RowData,"1");
               trigger text_align;
               trigger vert_text_align;

          else /if cmp($options['ROLAP'], "on");
                do /if cmp(proc_name, "Tabulate");
                    /*-- S0510551: conditionally emit alignments dck --*/
                    trigger text_align/if !cmp(just,'r');
                    trigger vert_text_align/if !cmp(vjust,'b');
                    break;
                else;
                    trigger just_and_vjust;
                done;

          else;
              trigger just_and_vjust;
          done;
      end;

      define event just_and_vjust;
           eval $colnum inputn(colstart, 'BEST');
           do /if ^cmp($justs[$colnum], just);
               /* putlog "JUSTS" ":" $justs[$colnum]  "  just" ":" just " value: " value;  */
               trigger text_align;
           done;
           break /if ^vjust;
           do /if ^cmp($vjusts[$colnum], vjust);
               /* putlog "VJUSTS" ":" $vjusts[$colnum]  "  vjust" ":" vjust "value: " value; */
               trigger vert_text_align;
           done;
       end;


       define event translated_to_string;
          /*
           putlog "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~" value ":" $column_types[$cell_count];
          */

           break /if ^value;
           break /if missing;
          do /if cmp(type, "double") | cmp(type, "int") | cmp(type, "bool") ;
              eval $translated inputn(value, 'BEST');
              eval $translated missing($translated);
              /*putlog "NUMERIC" ":" value "Missing" ":" $missing; */
          done;

          do /if $translated;
              do /if ^rawvalue;
                  putq' type="String"' /if ^cmp("String", $column_types[$cell_count]);
              done;
              unset $translated; /* S0700673 */
          done;
      end;

      define event csv_tag;
          start:
              unset $$csv_tag;
              open csv_tag;
              trigger stylename /if ^cmp(htmlclass, 'data');
              trigger missing_data;
              trigger do_justs;
              /* trigger text_align /if cmp($RowData,"1");*/

              /*
              do /if cmp($options['ROLAP'], "on");
                do /if cmp(proc_name, "Tabulate");
                    trigger text_align;
                    trigger vert_text_align;
                done;
              done;
              */

              trigger style_inline;

              do /if $do_formatted;
                  stop /if $doing_stacked_values;
                  stop /if $in_cube;

                  do /if ^cmp(type, $column_types[$cell_count]);
                      trigger print_type;
                      trigger print_format;
                  done;

                  trigger pre_post_text;

              /* doing unformatted - raw values. */
              else;

                  do /if rawvalue & ^$do_formatted;
                      set $string pretext prefix;
                      set $post_string posttext suffix;
                  else;
                      set $string pretext;
                      set $post_string posttext ;
                  done;

                  do /if pretext | prefix;
                      trigger clean_string;
                      putq ' prefix=' $string;
                      unset $string;
                  done;

                  do /if posttext | suffix;
                      set $string $post_string;
                      trigger clean_string;
                      putq ' suffix=' $string;
                      unset $string;
                      unset $post_string;
                  done;

                  /*----------------------------------------------eric-*/
                  /*-- Missing.  We'll be printing the formatted     --*/
                  /*-- value if we have one.  Set the type as a      --*/
                  /*-- string.                                       --*/
                  /*-------------------------------------------8Nov 04-*/
                  trigger translated_to_string;
                  do /if missing;
                      put ' type="String"' /if value;

                  /* not missing. life is good */
                  else;
                      do /if ^cmp(unformattedtype, $column_types[$cell_count]);
                          trigger print_unformattedtype;
                          trigger print_unformat;
                      /* S0700987 - Strip the saved value because I */
                      /* stored a blank when the value didn't exist. */
                      else /if ^cmp(sasformat, strip($col_formats[$cell_count])) |
                               ^cmp(scale, strip($col_scales[$cell_count])) |
                               ^cmp(precision, strip($col_precisions[$cell_count])) ;
                          trigger print_unformat;
                      else /if cmp($options['ROLAP'], "on");
                          do /if cmp(proc_name, "Tabulate");
                              trigger print_unformat;
                          done;
                      done;
                  done;
              done;
              do /if url;
                  eval $link_count $link_count + 1;
                  put ' action="link' $link_count '"';
              done;
              close;
              trigger do_link;

          finish:
              /*-- Don't emit <Value> tag unless attributes are present. --*/
              do /if $$csv_tag;
                  put '<Value';
                  put $$csv_tag;
                  put ' />';
              done;
              unset $$csv_tag;        /*-- Make sure                     --*/
          end;

          define event foot_cell;
              start:
                  /*trigger span;*/
                  put " " /if ^cmp(colstart, "1");
                  trigger table_label start /if $data_note;
                  put value;
                  /*
              finish:
                  trigger span;*/
          end;

      define event data;
          start:

              /* S0948617 - Fix compound IF - dch */
              do /if is_stacked;
                break /if cmp(event_name, 'data');
                break /if cmp(event_name, 'header');
              done;

              trigger foot_cell / breakif cmp(section,"foot");
              /*-------------------------------------------------------tph--*/
              /*-- Handle PROC REPORT's LINE statements.                  --*/
              /*---------------------------------------------------28Sep04--*/

              eval $cell_count $cell_count + 1;
              do /if colspan;
                  eval $colspan inputn(colspan, "3.") - 1;
              else;
                  unset $colspan;
              done;

              trigger csv_data /breakif $do_csv;

              trigger data_start;
          finish:

              /* S0948617 - Fix compound IF - dch */
              do /if is_stacked;
                break /if cmp(event_name, 'data');
                break /if cmp(event_name, 'header');
              done;

              trigger foot_cell / breakif cmp(section,"foot");

              do /if $colspan;
                  eval $cell_count $cell_count + $colspan;
              done;

              trigger csv_data /breakif $do_csv;

              do /if ^exists(empty) ;
                    put '</Value>' nl;
              done;
     end;

     define event print_unformattedtype;
          /*
          do /if cmp(unformattedtype, "double");
              put ' encoding="Base64"';
          else;
              put ' encoding="text"';
          done;
          */
          /* for proc freq.  It doesn't supply unformatted type */
          trigger print_type /breakif ^unformattedtype;

          do /if cmp(unformattedtype, "double") |
                 cmp(unformattedtype, "int") |
                 cmp(unformattedtype, "bool") ;
             put ' type="Numeric"';
          else;
              put ' type="String"';
              set $string_type "True";
          done;
     end;

     define event print_type;
          /*
          do /if cmp(type, "double");
              put ' encoding="Base64"';
          else;
              put ' encoding="text"';
          done;
          */
          do /if cmp(type, "double") | cmp(type, "int") | cmp(type, "bool") ;
              /*-------------------------------------------8Nov 04-*/
             put ' type="Numeric"';
          else;
             put ' type="String"';
             set $string_type "True";
          done;
      end;

      /*-----------------------------------------------tph--*/
      /*-- When scale and precision are '0' omit them.    --*/
      /*-- There must be EITHER a format name or scale    --*/
      /*-- must have a non-0 value (i.e. a w.d format).   --*/
      /*----------------------------------------------------*/
      define event print_format;
          unset $string_type /breakif $string_type;
          do  /if sasformat | ^cmp(scale, '0') | ^cmp(precision, '0');
              break /if ^any(sasformat, scale, precision);
              /*
              break /if ^cmp(type, "double") & ^cmp(type, "int") & ^cmp(type, "bool") ;*/
              put  ' SASFormat="';
              put  sasformat;
              put  scale /if !cmp(scale, '0');
              put  '.';
              put  precision /if !cmp(precision, '0');
              put  '"';
          done;
      end;

      define event print_unformat;
          unset $string_type /breakif $string_type;
          trigger print_format / breakif ^unformattedtype;
          do  /if sasformat | ^cmp(scale, '0') | ^cmp(precision, '0');
              break /if ^any(sasformat, scale, precision);
              break /if ^cmp(unformattedtype, "double") & ^cmp(unformattedtype, "int") & ^cmp(unformattedtype, "bool") ;
              put  ' SASFormat="';
              put  sasformat;
              put  scale /if !cmp(scale, '0');
              put  '.';
              put  precision /if !cmp(precision, '0');
              put  '"';
          done;
      end;

      define event clean_string;
          set $string  tranwrd($string, '&', '&amp;');
          set $string  tranwrd($string, '<', '&lt;');
          set $string  tranwrd($string, '>', '&gt;');
          set $string  tranwrd($string, '"', '&quot;');
          set $string  tranwrd($string, "'", '&apos;');
      end;

      define event clean_href;
          set $string  tranwrd($string, '&', '&amp;');
          set $string  tranwrd($string, '<', '&lt;');
          set $string  tranwrd($string, '>', '&gt;');
          set $string  tranwrd($string, "'", '&apos;');
      end;

      define event partial_clean_string;
          set $string  tranwrd($string, '<', '&lt;');
          set $string  tranwrd($string, '>', '&gt;');
          set $string  tranwrd($string, '"', '&quot;');
          set $string  tranwrd($string, "'", '&apos;');
      end;

     define event csv_data;
         start:
             /*-------------------------------------------------------eric-*/
             /*-- Remove this if we want to start outputing table footers.-*/
             /*----------------------------------------------------28Jan03-*/
             unset $value;
             trigger table_label / breakif cmp(section,"foot");


             trigger csv_tag;
             put ", " /if $cell_count > 1;

             break / if empty;

             do /if $do_formatted;
                 put '"';
                 set $close_quote "True";
                 set $type type;  /* save the type for the finish */
                 /*set $value tranwrd(value, '<Br/>', ' ');*/
                 put tranwrd(value, '&quot;', '""');
             else;
                 do /if cmp(unformattedtype, 'string') | cmp(unformattedtype, 'char');
                     set $close_quote "True";
                     put '"';
                 done;
                 set $type unformattedtype;
                 do /if rawvalue;
                     put rawvalue ;
                 else;
                     /*------------------------------------------eric-*/
                     /*-- Print the formatted value as a string if  --*/
                     /*-- it's missing.  I'm not really sure we'll  --*/
                     /*-- ever really want unformatted values in    --*/
                     /*-- this case.                                --*/
                     /*---------------------------------------8Nov 04-*/
                     do /if unformattedvalue;
                         /*put  unformattedvalue ;   CHANGE BACK when WE GET CDATA*/
                         set $value tranwrd(value, '&quot;', '""');
                         do /if missing;
                             putq $value /if missing;
                         else;
                             put $value;
                         done;
                     else;
                        set $value tranwrd(value, '&quot;', '"');
                        set $value tranwrd($value, '<Br/>', ' ');
                        put tranwrd($value, '"', '""');
                     done;
                 done;
             done;

         finish:
             /*-------------------------------------------------------eric-*/
             /*-- Remove this if we want to start outputing table footers.-*/
             /*----------------------------------------------------28Jan03-*/
             trigger table_label / breakif cmp(section,"foot");

             break /if empty;

             /* Quote all formatted values in csv */
             do /if $close_quote;
                 unset $close_quote;
                 put '"';
             done;

             flush;  /* without this it could come out in the wrong place */

             trigger csv_tag;
      end;


      define event do_link;
          break /if !url;
          open Actions;
             put '<URLAction name="link' $link_count '">' nl;
             put '<URLValue>';
             put url;
             put '</URLValue>' nl;
             put '</URLAction>' nl;
          close;
      end;

      define event do_actions;
          put "<Actions>" nl;
          put $$Actions;
          put "</Actions>" nl;
      end;

      define event data_start;
          put '<Value';
          trigger missing_data;
          trigger stylename /if ^cmp(htmlclass, 'data');
          trigger text_align / if cmp($RowData,"1");
          trigger style_inline ;
          do /if ^cmp(type, $column_types[$cell_count]);
             stop /if $doing_stacked_values;
             stop /if $in_cube;
             trigger print_type;
             trigger print_format;
          done;
          put ' /' / if exists(empty);
          put '>';
          put nl / breakif exists(empty);
          put value;
      end;


      define event table_body;
          start:
            do /if $do_csv;
                break /if $in_csv;
                put '<CSV>' nl;
                /* put '<![CDATA[' nl; */
                set $in_csv "true";
                flush;
            else;
                break /if $in_valueslist;
                put '<ValuesList>' nl;
                set $in_valueslist "true";
            done;
      end;

      /*---------------------------------------------------------eric-*/
      /*-- For tables and images.                                   --*/
      /*------------------------------------------------------26Aug03-*/
      define event griddedconstraint;
         put '<GriddedConstraint';
         put ' halign="center"' / if cmp(just, 'c');
         put ' halign="left"' / if cmp(just, 'l');
         put ' halign="right"' / if cmp(just, 'r');
         put ' fill="horizontal"';   /* added per dabail 3/10/04 - eric */
         put ' />' nl;
      end;

      define event fit_to_container;
        trigger griddedconstraint;
      end;

      define event init_border_style;
            /*Frame Value     border-style Value*/
            set $borderStyles["VOID"]          "none";
            set $borderStyles["ABOVE"]         "solid none none none";
            set $borderStyles["BELOW"]         "none none solid none";
            set $borderStyles["HSIDES"]        "solid none";
            set $borderStyles["VSIDES"]        "none solid";
            set $borderStyles["LHS"]           "none none none solid";
            set $borderStyles["RHS"]           "none solid none none";
            set $borderStyles["BOX"]           "solid";
            set $borderStyles["BORDER"]        "solid";
      end;

      define event view_table;
         start:
           open view;
           trigger text finish;
           trigger page_headers finish;
           trigger proctitleheaders finish;
           trigger bodysection;
           trigger pre_post_image start;
           trigger pre_post_textheader start;
           put "<Table";
           putq ' data=' anchor;
           trigger output_name;
           trigger putall;
           trigger stylename;
           trigger page_break_before;
           trigger style_inline;
           do /if cellspacing;
               set $size_unit cellspacing;
               trigger parse_size_unit;
               put ' cellSpacing="' $size_unit_size $size_unit_unit '"';
               unset $size_unit_size;
               unset $size_unit_unit;
           done;
           do /if cellpadding;
               set $size_unit cellpadding;
               trigger parse_size_unit;
               put ' cellPadding="' $size_unit_size $size_unit_unit '"';
               unset $size_unit_size;
               unset $size_unit_unit;
           done;
           do /if !any(borderstyle, borderwidth, bordercolor);
               putq  ' style:borderStyle=' $borderStyles[frame];
           done;
           put '>' nl;
           trigger griddedconstraint;
           ndent;
           close;
         finish:
           open view;
           xdent;
           put "</Table>" nl;
           trigger pre_post_textheader finish;
           trigger pre_post_image finish;
           close;
      end;

      define event table;
         start:
           unset $column_types;
           unset $col_formats;
           unset $col_scales;
           unset $col_precisions;
           put '<IQData type="2DEmbedded"' ;
           putq ' name=' anchor ;
           trigger putall;
           put '>' nl;
           ndent;
           put "<EmbeddedData>" nl;
           ndent;
           trigger view_table;
         finish:
            do /if $do_csv;
                /* put ']]>' nl; */
                put '</CSV>' nl;
                unset $in_csv ;
            else;
                put '</ValuesList>' nl;
                unset $in_valueslist ;
            done;
           trigger view_table;
           xdent;
           put "</EmbeddedData>" nl;
           xdent;
           put "</IQData>" nl;
      end;

      define event table_headers;
         start:
             unset $varnames;
             unset $justs;
             unset $vjusts;
             /*putlog "-----------New Table-----------" anchor;*/
            put '<Variables>' nl;
            trigger view_columns;
         finish:
            trigger view_columns;
            put '</Variables>' nl;
      end;

      define event header_spec;
         start:
           open view;
           put '<Columns order="specified">' nl;
           ndent;
           trigger label;
           close;
         finish:
           open view;
           xdent;
           put '</Columns>' nl;
           close;
      end;

      /* Non-spanning column label. Report generates empty Labels to  */
      /* suppress the default use of the column name as a label.      */
      define event col_header_label;
         start:
            open view;
            put '<Label';
            trigger event_attr;
            trigger style_name_align;
            trigger vert_text_align;
            trigger preformatted;
            put ' isVisible="false"' / if exists(empty);
            put ' isStacked="true"'  / if is_stacked;
            put ' isStacked="false"' / if !is_stacked;
            put ' position="top"';
            put '/>' nl              / if exists(empty);
            close                    / if exists(empty);
            break                    / if exists(empty);
            put '>';
              set $string pretext;
              trigger clean_string;
              put $string;
              unset $string;

              set $string value;
              /*trigger clean_string;*/
              put  $string;
              unset $string;

              set $string posttext;
              trigger clean_string;
              put $string;
              unset $string;
            /* Anything that happens between here and  */
            /* the finish will be written to the view. */
            /* Required for PRINT vertical headers.    */
         finish:
            open view;  /* sometimes it gets closed */
            break / if exists(empty);
            put '</Label>' nl;
            close;
     end;

     define event event_attr;
         /*putq " eventName=" event_name;*/
     end;

      define event label;
        /* It is critically important that the <Label>text</Label> */
        /* sequence not have any extra white space between the     */
        /* tags and the text!                                      */
        put  '<Label';
        trigger event_attr;
        put  ' isVisible="false"' / if !exists(value);
        put  ' isStacked="true"'  / if is_stacked;
        put  ' isStacked="false"' / if !is_stacked;
        putq ' position=' position;
        trigger stylename;
        trigger text_align;
        trigger vert_text_align;
        trigger style_inline;
        trigger preformatted;
          put '>';
          /*
        set $string value;
        trigger clean_string;
          put  $string;
          */
          trigger pre_post_text_value start;
        put value;
        trigger pre_post_text_value finish;
        put '</Label>' nl;
      end;

      define event col_header_spec;
         start:
            trigger header_spec;
         finish:
            trigger header_spec;
      end;

      define event sub_header_stacked_colspec_entry;
          start:
              trigger view_column;
              trigger sr_variable;
          finish:
              trigger view_column;
      end;

      define event sub_header_colspec;
        start:
          do /if is_stacked;
             set $do_stacked_view "True";
             trigger stacked_view_column;
          else;
             trigger view_column;
             trigger sr_variable;
          done;
        finish:
          do /if $do_stacked_view;;
             unset $do_stacked_view;
             trigger stacked_view_column;
          else;
             trigger view_column;
          done;
      end;

      define event sr_variable;
          /*
          putlog "VARIABLE just" ":" just;
          putlog "VARIABLE vjust" ":" vjust;
          */
          do /if cmp(just, 'd');
              set $justs[] 'r';
          else;
              set $justs[] just;
          done;
          do /if vjust;
              set $vjusts[] vjust;
          else;
              set $vjusts[] 'm';
          done;

          do /if sasformat;
              set $col_formats[] sasformat;
          else;
              set $col_formats[] ' ';
          done;

          do /if scale;
              set $col_scales[] scale;
          else;
              set $col_scales[] ' ';
          done;

          do /if precision;
              set $col_precisions[] precision;
          else;
              set $col_precisions[] ' ';
          done;

          put  '<Variable';
          /*-------------------------------------------------------eric-*/
          /*-- change these 1st three back to numeric when BIP is     --*/
          /*-- ready to use them as numerics...                       --*/
          /*-- Apparently formatted will forever be strings...        --*/
          /*-- Changing back to String.    08/10/05                   --*/
          /*----------------------------------------------------5Feb 03-*/


          do /if $do_csv;

              do /if $do_formatted;
                  put  ' encoding="text"';
                  do /if cmp(type, "double") | cmp(type, "int") | cmp(type, "bool") ;
                     put ' type="String"';
                  else;
                     put ' type="String"';
                  done;
                  set $column_types[] type;

              else;
                  trigger print_unformat;
                  set $column_types[] unformattedtype;

                  do /if cmp(unformattedtype, "double");
                      put ' encoding="Base64" type="Numeric"';
                  else /if cmp(unformattedtype, "int") | cmp(unformattedtype, "bool") ;
                      put ' encoding="text" type="Numeric"';
                  else;
                      put ' encoding="text" type="String"';
                  done;
              done;
          else;
              put  ' encoding="Text"';
              put  ' type=';
              put  '"String"' / if cmp(type, "double");
              put  '"String"' / if cmp(type, "int");
              put  '"String"' / if cmp(type, "bool");
              put  '"String"'  / if cmp(type, "char");
              put  '"String"'  / if cmp(type, "string");
              put  '"String"'  / if cmp(type, "unknown");
          done;
          do / if cmp(proc_name,"Print");
              set $string name;
              trigger clean_string;
              putq ' name=' $string;
              unset $string;
          else;
              trigger alt_colid  / if !cmp(proc_name, "Print");
          done;
          put  ' />' nl;
      end;

/*
      define event alt_colid;
          unset $name;
          set $name data_name /if data_name;
          set $name ref_id /if ^$name;
          set $name name /if ^$name;
          set $name "var" col_id /if ^$name;

          do /if data_name;
              set $name data_name;
          else /if ref_id;
              set $name "!junk";
              set $name ref_id;
          else /if name;
              set $name "!junk";
              set $name name;
          else;
              set $name "var" col_id /if ^$name;
          done;

          do /if $colnames[$name];
             eval $colnames[$name] $colnames[$name] +1;
          else;
             eval $colnames[$name] 1;
          done;

          put ' name="' ;
          put $name ;
          do /if $colnames[$name] ^= 1;
              put "_" $colnames[$name];
          done;
          put '"'    ;
      end;
*/

/*
      define event alt_colid;
          put  ' name="var'  ;
          put  col_id        ;
          put  '"'           ;
      end;
*/

      define event span_group;
      end;
      define event span_header_colspec;
      end;

      define event colspecs;
        start:
          break /if  !cmp(data_viewer, 'Batch');
          put '<Variables>' nl;
          trigger header_spec;
        finish:
          break /if  !cmp(data_viewer, 'Batch');
          put '</Variables>' nl;
          trigger header_spec;
          trigger table_body;
      end;

      define event colgroup;
      end;

      define event colspec_entry;
        start:
          break /if  !cmp(data_viewer, 'Batch');
          trigger sub_header_colspec;
        finish:
          break /if  !cmp(data_viewer, 'Batch');
          trigger sub_header_colspec;
    end;

    define event sub_colspec_header;
    end;

      define event view_columns;
         start:
           open view;
           put '<Columns order="specified"> ' nl;
           ndent;
           close;
         finish:
           open view;
           xdent;
           put '</Columns> ' nl;
           close;
      end;

      define event view_column;
        start:
          open view;
          put '<Column';
          trigger stylename;
      /*  Disable until consumers ready for raw data + formats  */
      /*  putq ' SASFormat=' sasformat / if cmp(genformat,"1"); */
      /*  putq ' SASFormat=' sasformat;                         */
      /*  putq " encoding=" dataencoding;                       */
          do /if  cmp(proc_name,"Print");
              set $string name;
              trigger clean_string;
              putq ' variable=' $string;
              unset $string;
          else;
            trigger altvarid;
          done;
          put '>' nl ;
          ndent ;
          put '<Cell ' ;
          trigger text_align;
          putq " style:style=" htmlclass;
          trigger preformatted /if asis;
          put '/>' nl;
          do / if cmp( data_viewer, 'Tree' );
            putl '<Label isVisible="false"></Label>';
          done;
          close;
        finish:
          open view;
          xdent;
          put '</Column>' nl;
          close;
      end;

      define event stacked_view_column;
        start:
          open view;
          put '<Columns order="specified"' ;
          trigger stylename;
      /*  Disable until consumers ready for raw data + formats  */
      /*  putq ' SASFormat=' sasformat / if cmp(genformat,"1"); */
      /*  putq ' SASFormat=' sasformat;                         */
      /*  putq " encoding=" dataencoding;                       */
          putq ' isStacked="true"' ;
          put '>' nl ;
          ndent ;
          close;
        finish:
          open view;
          xdent;
          put '</Columns>' nl;
          close;
      end;


      define event altvarid;
          set $name data_name /if data_name;
          /*set $name ref_id /if ^$name;*/
          set $name name /if ^data_name;
          set $name "var" col_id /if ^$name;

          do /if $varnames[$name];
             eval $varnames[$name] $varnames[$name] +1;
          else;
             eval $varnames[$name] 1;
          done;

          put ' variable="' ;
          set $string $name;
          trigger clean_string;

          put $string ;
          unset $string;

          do /if $varnames[$name] ^= 1;
              put "_" $varnames[$name];
          done;
          put '"'    ;
      end;

      define event alt_colid;
          put ' name="' ;
          set $string $name;
          trigger clean_string;
          put $string ;
          unset $string;
          do /if $varnames[$name] ^= 1;
              put "_" $varnames[$name];
          done;
          put '"'    ;
      end;

/*
      define event alt_colid;
          unset $name;
          set $name data_name /if data_name;
          set $name ref_id /if ^$name;
          set $name name /if ^$name;
          set $name "var" col_id /if ^$name;

          do /if data_name;
              set $name data_name;
          else /if ref_id;
              set $name "!junk";
              set $name ref_id;
          else /if name;
              set $name "!junk";
              set $name name;
          else;
              set $name "var" col_id /if ^$name;
          done;

          do /if $colnames[$name];
             eval $colnames[$name] $colnames[$name] +1;
          else;
             eval $colnames[$name] 1;
          done;

          put ' name="' ;
          put $name ;
          do /if $colnames[$name] ^= 1;
              put "_" $colnames[$name];
          done;
          put '"'    ;
      end;
*/

/*
      define event alt_colid;
          put  ' name="var'  ;
          put  col_id        ;
          put  '"'           ;
      end;
*/

      define event table_head;
         start:
            block data;
            block row;
            block header;
         finish:
            unblock data;
            unblock row;
            unblock header;
     end;

     /*
      define event table_foot;
         start:
            block data;
            block row;
            block breakline;
         finish:
            unblock data;
            unblock row;
            unblock breakline;
     end;
     */

      define event datetime;
         break;  /* no dates.  It's in the header now */
         put '<!-- no content -->' nl / if !exists($do_date);
         break /if !exists($do_date);
         unset $do_date;
         put '<Date style:textAlign="right"';
         put ' showDate="false" showTime="false"' / if cmp(getoption('DATE'),'NODATE');
         put '>';
         put javadate 'T' javatime;
         put '</Date>' nl;
      end;

      define event text;
          start:
              break /if cmp($text_just, just) & cmp($text_vjust, vjust);

              /*
              do /if cmp($text_just, just);
                  do /if $text_vjust;
                      do /if cmp($text_vjust, vjust);
                          break;
                      done;
                  else;
                      break;
                  done;
              done;
              */

              trigger text finish;

              set $text_just just;
              set $text_vjust vjust;
              set $in_text "True";
              /*eval $proc_title_count 0; */

              put '<Text';
              trigger output_name;
              trigger style_name_align;
              trigger vert_text_align;
              put '>' nl;
              ndent;
              trigger fit_to_container;
              trigger datetime;
          finish:
              break /if ^$in_text;
              put '</Text>' nl;
              unset $text_just;
              unset $text_vjust;
              unset $in_text;
      end;

      define event textheader;
         start:
            trigger text;
            put '<Span';
            trigger stylename;
            trigger page_break_before;
            trigger style_inline;
            put '>';
         finish:
            /*
            do /if cmp(event_name, 'proc_title');
                do /if $proc_title_count;
                    put '<Br/>' /if ^$in_proc_note_group;
                done;
            done;
            */
            put '</Span>' nl;
            xdent;
      end;

      /* Stand-alone <Text> not associated with */
      /* a title. Used by PROC FREQ crosstabs.  */
      /* and paragraph (ods text=)              */
      define event body_text;
          start:
              open view;
              put '<Text';
              trigger output_name;
              trigger style_name_align;
              put '>' nl;
              trigger fit_to_container;
              ndent;
              put '<Span>';
              /* Do not add a newline after '>'. There can be  */
              /* no white space between the tags and the text. */
              put value;
          finish:
              put '</Span>' nl;
              xdent;
              put '</Text>' nl;
              close;
      end;
      /*
<Footer style:style="Body" style:paddingBottom="10px"
style:textAlign="center">
  <GriddedLayout rows="2" rowGutter="10 px" columnGutter="10 px"/>
  <Text style:style="SystemFooter" style:textAlign="left">
    <GriddedConstraint row="1" column="1" halign="left"/>
    <Span style:style="SystemFooter"> ABC Engineering, Inc</Span>
  </Text>
  <Text style:style="Normal" style:fontFamily="swiss"
style:fontStyle="normal" style:width="50%" style:textAlign="left"
style:fontSize="small" style:fontWeight="normal">
    <GriddedConstraint row="2" column="1" halign="left"/>
    <Span style:style="Normal" style:width="50%"> January 1998</Span>
  </Text>
  <Text style:style="Normal" style:fontFamily="swiss"
style:fontStyle="normal" style:width="50%" style:textAlign="right"
style:fontSize="small" style:fontWeight="normal">
    <GriddedConstraint row="2" column="2" halign="right"/>
    <Span style:style="Normal" style:fontFamily="swiss"
style:fontStyle="normal" style:width="50%" style:fontSize="small"
style:fontWeight="normal">GSLTEXTS </Span>
  </Text>
      </Footer>
      */


        define event title_container_specs;
            start:
                eval $title_columns 0;
            finish:
                open view;
                trigger footer start /if $in_system_footer;
                do /if ^$in_system_footer;
                    trigger page_headers start /if ^$in_system_footer;
                done;
                close;
        end;

        /*---------------------------------------------------eric-*/
        /*-- Add them up because the colcount on the            --*/
        /*-- title_container_spec is wrong.                     --*/
        /*------------------------------------------------20Oct04-*/

        define event title_container_spec;
            eval $title_columns $title_columns + 1;
        end;

        define event title_container_row;
            eval $title_column_count 1;
        end;

        define event footer_container_row;
            trigger title_container_row;
        end;

        define event system_title_group;
            finish:
                open view;
                  unset $do_date;
                  xdent;
                close;
        end;

        define event proc_title_group;
        start:
            eval $proc_title_count 0;
            open view;
            trigger bodysection finish;
            eval $title_columns 1;
            trigger page_headers;
            close;
        /*
        finish:
            eval $proc_title_count 0;*/
        end;

        define event proc_title_redef_group;
            trigger proc_title_group;
        end;

/*
        define event leaf;
            start:
              open view;
              trigger bodysection start;
              close;
        end;
*/
        define event output;
            start:
                /* Remove following message when S0463294 is fixed. */
                do / if cmp( data_viewer, 'Null' );
                   putlog 'ERROR: ODS Report Writing is preproduction in this release. The SASReport tagset is not supported with ODS Report Writing and execution has stopped.';
                   trigger output finish;
                done;
                set $output_label output_label;
              open view;
              trigger bodysection start;
                close;
            finish:
                unset $output_label;
        end;

        define event info;
            start:
                open view;
                put '<' event_name ;
                trigger putall;
                put '>' nl;
                close;
            finish:
                open view;
                put '<' event_name '/>' nl;
                close;
        end;


        define event putall;
            /*
            putq ' Name=' Name;
            putq ' Label=' label;
            putq ' OutPutName=' output_name;
            putq ' Value=' value;
            */
            /*
            putvars event ' ' _name_ '="' _value_ '"';
            putvars mem ' ' _name_ '="' _value_ '"';
            */
        end;

        define event bodysection;
            start:
                trigger page_headers finish;
                trigger proctitleheaders finish;
                break /if $inbody;
                set $inbody "True";
                put "<Body" ;
                put ' style:paddingBottom="10px"';
                put ">" nl;
                ndent;
                put '<GriddedLayout rowGutter="10px"/>' nl;
           finish:
               break /if !exists($inbody);
                xdent;
                trigger text;  /* in case a proc put a title after the table */
                put "</Body>" nl;
                set $section_had_body "True";
                unset $inbody;
                unset $body_level;
       end;

        define event page_headers;
           start:
              break /if $inbody;
              break /if $in_page_headers;
              do /if $section_had_body;
                 trigger section finish;
                 trigger section start;
              done;
              do/if $layout_nesting_level eq 0;
                put '<Header style:paddingBottom="10px"';
              else;
                put '<Container';
              done;
              putl '>';
              put '<GriddedLayout';
              putq ' columns=' $title_columns;
              put ' rowGutter="10px" columnGutter="10px"/>' nl;
              ndent;
              set $in_page_headers 'true';
           finish:
              break /if !exists($in_page_headers);
              trigger text;
              do/if $layout_nesting_level eq 0;
                putl '</Header>';
              else;
                putl '</Container>';
              done;
              unset $in_page_headers;
        end; /* page_headers */

        define event proc;
            start:
                eval $output_count 0;
                open view;
                /*trigger bodysection;*/
                trigger proc_count;
                trigger proc_section;
                close;
            finish:
                open view;
                trigger text finish;
                trigger procnoteheaders finish;
                trigger text finish;
                trigger proctitleheaders finish;

                trigger bodysection;
                trigger procnotesection finish;
                trigger proctitlesection finish;
                trigger proc_section;
                close;
                unset $current_proc;
        end;

        define event proc_count;
            do /if $proc_count[name];
                eval $proc_count[name] $proc_count[name] + 1;
            else;
                eval $proc_count[name] 1;
            done;
        end;

       define event proc_section;
            start:
                set $current_proc name '_' $proc_count[name];
                break/if $layout_nesting_level > 0;
                putq '<Section' ' name=' $current_proc ' style:pageBreakAfter="always">' nl;
                trigger pagesetup;
                trigger pagebreak_section;
            finish:
                break/if $layout_nesting_level > 0;
                unset $section_had_body;
                trigger pagebreak_section;
                put '</Section>' nl;
        end; /* proc_section */

        define event pagebreak_section;
            start:
               break /if $pagebreak_section;
               set $pagebreak_section "True";
               putq '<Section style:pageBreakAfter="always">' nl;
               ndent;
        finish:
               break /if ^$pagebreak_section;
               xdent;
               unset $pagebreak_section;
               unset $section_had_body;
               trigger page_headers;
               trigger procnotesection;
               trigger proctitlesection;
               putq '</Section>' nl;
        end;

        define event pagebreak;
            open view;
            trigger text finish;
            trigger procnoteheaders finish;
            trigger text finish;
            trigger proctitleheaders finish;
            trigger bodysection finish;
            trigger pagebreak_section finish;
            trigger pagebreak_section start;
            close;
        end;

        /*
        define event title_setup_format_section;
             put value;
        end;
        */

        define event title_format_section;
            put '<Span';
            trigger style_name_align;
            put '>';
            trigger pre_post_text_value start;
            put value;
            trigger pre_post_text_value finish;
            put '</Span>' nl;
        end;


        define event system_title;
        start:
            open view;
            trigger title_textheader;
            trigger pre_post_text_value start;
            put value;
            trigger pre_post_text_value finish;
        finish:
            trigger title_textheader;
            close;
        end;

      define event title_griddedconstraint;
         put '<GriddedConstraint';
         put ' halign="center"' / if cmp(just, 'c');
         put ' halign="left"' / if cmp(just, 'l');
         put ' halign="right"' / if cmp(just, 'r');
         put ' fill="horizontal"';
         putq ' columnSpan=' colspan;
          putq ' column=' $title_column_count;  /* the only difference from the regular event */
         do /if colspan;
             eval $title_column_count $title_column_count + inputn(colspan, 'BEST');
         else;
             eval $title_column_count $title_column_count + 1;
         done;
         put ' />' nl;
      end;

      define event title_text;
          start:
              /*break /if cmp($text_just, just) & cmp($text_vjust, vjust);

              trigger text finish; */

              trigger text finish;

              set $text_just just;
              set $text_vjust vjust;
              set $in_text "True";

              put '<Text';
              trigger output_name;
              trigger style_name_align;
              trigger vert_text_align;
              /* S0667327 - dch */
              do /if url;
                  eval $link_count $link_count + 1;
                  put ' action="link' $link_count '"';
              done;
              put '>' nl;
              ndent;
              trigger title_griddedconstraint;
          finish:
              break /if ^$in_text;
              put '</Text>' nl;
              unset $text_just;
              unset $text_vjust;
              unset $in_text;
              trigger do_link; /* S0667327 - dch */
      end;

      define event title_textheader;
         start:
            trigger title_text;
            put '<Span';
            trigger style_name_align;
            put '>';
         finish:
            put '</Span>' nl;
            trigger title_text;
      end;

        define event system_footer_group;
          /*start:
              set $in_system_footer 'True';*/
          finish:
              trigger footer;
        end;

        define event footer_format_section;
            put value;
        end;

       define event footer;
            start:
                break /if $in_system_footer;
                open view;
                trigger bodysection finish;
                set $in_system_footer 'True';
                do/if $layout_nesting_level eq 0;
                  put '<Footer';
                else;
                  put '<Container';
                done;
                put ' style:paddingBottom="10px"';
                trigger style_name_align;
                put '>' nl;
                put '<GriddedLayout';
                putq ' columns=' $title_columns;
                put ' rowGutter="10px" columnGutter="10px"/>' nl;
                /* colcount indicates the number of parts in a title */
                /*putq '<GriddedLayout columns=' colcount '/>' nl;   */
                close;
             finish:
                break /if ^$in_system_footer;
                unset $in_system_footer;
                open view;
                trigger text;
                do/if $layout_nesting_level eq 0;
                  putl '</Footer>';
                else;
                  putl '</Container>';
                done;
                close;
        end; /* footer */

        define event system_footer;
            start:
                trigger footer;
                open view;
                trigger title_textheader;
                trigger pre_post_text_value start;
                put value;
                trigger pre_post_text_value finish;
            finish:
                trigger title_textheader;
                close;
        end;

        define event pre_post_text_value;
            start:
                do /if pretext;
                    set $string pretext;
                    trigger clean_string;
                    put  $string ' ';
                    unset $string;
                done;
            finish:
                do /if posttext;
                    set $string posttext;
                    trigger clean_string;
                    put  $string ' ';
                    unset $string;
                done;
        end;

        define event byline;
            open view;
             trigger text finish;
             do /if ^$proctitlesection;
                    do /if $in_page_headers;
                        put '</Header>' nl;
                        unset $in_page_headers;
                    done;
                else /if $inbody;
                        trigger bodysection finish;
                        trigger proctitlesection finish;
                done;
                trigger proctitlesection start;

            do /if ^any($in_page_headers, $proctitleheaders, $procnoteheaders);
                set $close_headers 'yes';
                trigger procnoteheaders;
            done;

            trigger textheader;
            trigger pre_post_text_value start;
            put value;
            trigger pre_post_text_value finish;
            trigger textheader finish;
            trigger text finish;
            do /if $close_headers;
                trigger procnoteheaders finish;
                unset $close_headers;
            done;
            close;
        end;

        define event proc_title_row;
            start:
                open view;
                put '<Br/>' /if $proc_title_count;
                close;
        end;

        define event proc_title;
            open view;
                do /if ^$in_proc_note_group;

                    do /if ^$proctitlesection;
                        do /if $in_page_headers;
                            put '</Header>' nl;
                            unset $in_page_headers;
                        done;
                    else /if $inbody;
                            trigger bodysection finish;
                            trigger proctitlesection finish;
                    done;
                    trigger proctitlesection start;

                done;

            trigger textheader;
            trigger pre_post_text_value start;
            put value;
            trigger pre_post_text_value finish;
            trigger textheader finish;
            close;
            eval $proc_title_count $proc_title_count + 1;
        end;

        /*
        define event proc_title_redef;
            open view;
            put '<Text ';
            trigger style_name_align;
            put '>';
            trigger fit_to_container;
            put '<Span';
            putq ' topOfPage="True"';
            put '>' nl;
            trigger pre_post_text_value start;
            put value;
            trigger pre_post_text_value finish;
            put '</Span>' nl;
            put '</Text>' nl;
            close;
        end;
        */

        define event note;
            open view;
            trigger textheader;
            put 'Note: ' value;
            trigger textheader finish;
            trigger text finish;
            close;
        end;
        define event error;
            open view;
            trigger textheader;
            put 'Error: ' value;
            trigger textheader finish;
            trigger text finish;
            close;
        end;
        define event fatal;
            open view;
            trigger textheader;
            put 'Fatal: ' value;
            trigger textheader finish;
            trigger text finish;
            close;
        end;
        define event warning;
            open view;
            trigger textheader;
            put 'Warning: ' value;
            trigger textheader finish;
            trigger text finish;
            close;
        end;

        /*-------------------------------------------------------eric-*/
        /*-- Only do sections for branches that are bygroups...     --*/
        /*----------------------------------------------------17May04-*/
        define event bygroup;
            start:
                set $in_bygroup "True";
            finish:
                unset $in_bygroup;
        end;

        define event branch;
            start:
                set $string value;
                /*trigger clean_string;*/
                set $branch_stack[] $string;
                set $branch_value value /if $in_bygroup;
                unset $string;
            finish:
                unset $branch_stack[-1];
        end;

        define event proctitleheaders;
            start:
                break /if $procnoteheaders;
                break /if $proctitleheaders;
                put '<Header style:paddingBottom="10px">' nl;
                put '<GriddedLayout rowGutter="10px" columnGutter="10px" />' nl;
                set $proctitleheaders "True";
            finish:
                break /if $procnoteheaders;
                break /if ^$proctitleheaders;
                unset $proctitleheaders;
                trigger text;
                put '</Header>' nl;
        end;

        define event procnoteheaders;
            start:
                break /if $procnoteheaders;
                put '<Header style:paddingBottom="10px">' nl;
                put '<GriddedLayout rowGutter="10px" columnGutter="10px" />' nl;
                set $procnoteheaders "True";
            finish:
                break /if ^$procnoteheaders;
                unset $procnoteheaders;
                trigger text;
                put '</Header>' nl;
        end;

        define event proc_note_group;
            start:
                set $in_proc_note_group "true";
                open view;
                trigger bodysection finish;
                trigger procnotesection finish;
                trigger proctitleheaders finish;
                trigger page_headers finish;
                trigger procnotesection start;
                trigger procnoteheaders start;
                close;
            finish:
                open view;
                trigger procnoteheaders finish;
                unset $in_proc_note_group;
                close;
        end;

        /*
        define event proc_title_group;
            start:
                trigger proctitlesection finish;
                trigger proctitlesection start;
                trigger proctitleheaders start;
            finish:
                trigger proctitleheaders finish;
        end;
        */

        define event procnotesection;
            start:
               break /if $procnotesection;
               put "<Section";
                put ">" nl;
               trigger procnoteheaders start;
               set $procnotesection "True";
               ndent;
           finish:
               break /if ^$procnotesection;
               xdent;
               unset $procnotesection;
               putq "</Section>" nl;
        end;

        define event proctitlesection;
            start:
               break /if $proctitlesection;
               put "<Section";
                put ">" nl;

               trigger proctitleheaders start;
               set $proctitlesection "True";
               ndent;
           finish:
               break /if ^$proctitlesection;
               xdent;
               unset $proctitlesection;
               putq "</Section>" nl;
        end;

        define event section;
           start:
               put "<Section";

               /*-----------------------------------------------------eric-*/
               /*-- put the html link in.  But only on the first section.--*/
               /*--------------------------------------------------9Jul 03-*/
               do /if !$ods_link;
                   putq " ods:html=" tagset_alias;
                   set $ods_link "TRUE";
               done;

               /* when triggered from leaf event the name     */
               /* variable is preferred over the value        */
               /* variable as the value of the name attribute */
               putq " name=" name;
               putq " name=" value / if !exists(name);
               put ">" nl;
               ndent;
           finish:
               xdent;
               unset $section_had_body;
               trigger page_headers;
               putq "</Section>" nl;
        end;

        /* Generate <Image /> tag for graph procs. */
      define event image;
        start:
            pure_style;
            open view;
            trigger text finish;
            trigger page_headers finish;
            put '<Image';
            /* S0731786 - Remove this for now and replace it with a call to bygroup_slice*/
            /* due to MVS regression. See what changed in Graph later */
            /* trigger alt_output_name; */
            trigger bygroup_slice;

            trigger text_align;

            trigger style_inline;  /* Added to get height and width etc. */
            /*put $$graph_border;*/

            set $string alt;
            trigger clean_string;
            put ' alternateText="' $string '"' /if $string;
            unset $string;

            put ' file="' url '"';
            put ' usemap="#' @CLIENTMAP;
            put ' usemap="#' NAME / if !exists(@CLIENTMAP);
            put '"' / if any(@CLIENTMAP , NAME);
            put ' >' nl;
            trigger griddedconstraint;
            close;
        finish:
            open view;
            put $$image_map;
            put '</Image>' nl;
            close;
            unset $$image_map;
    end; /* image */

      define event pre_post_textheader;
          start:
              /*putlog "PREPOST START " event_name " PRE: " pretext " POST: " posttext;   */
           set $posttext posttext;
           break /if ^pretext;

           set $prepost "true";
           trigger textheader start;
            set $string pretext;
            trigger clean_string;
              put  $string ;
              unset $string;
           trigger textheader finish;
           trigger text finish;
           unset $prepost;
        finish:
           /*putlog "PREPOST FINISH " event_name " PRE: " pretext " POST: " posttext;   */
           break /if ^$posttext;
           set $prepost "true";
           trigger textheader start;
            set $string $posttext;
            trigger clean_string;
            put  $string ;
            unset $string;
            unset $posttext;
           trigger textheader finish;
           trigger text finish;
           unset $prepost;
      end;

      define event pre_post_image;
          start:
            set $postimage postimage;
            break /if ^preimage;
            put '<Image';
            put ' file="' preimage '"';
            put ' >' nl;
            put '</Image>' nl;
        finish:
            break /if ^$postimage;
            put '<Image';
            put ' file="' $postimage '"';
            put ' >' nl;
            put '</Image>' nl;
            unset $postimage;
    end;

      define event map_coordinate;
          put ' coords="' / if !exists($coord_started);
          put ',' / if exists($coord_started);
          set $coord_started "true";
          put coordinate;
      end;

      define event map;
        start:
          open image_map;
          set $image_map "True";
          putl "<Map>";
        finish:
          putl "</Map>";
          unset $image_map;
          close;
    end;

      define event map_area;
          start:
          put "<Area";
          putq " shape=" shape;
          set $string value;
          trigger clean_href;
          put " " $string;

          do /if any(url, htmlid);
              set $string url;
              trigger clean_string;
              put ' href="';
              put $string '#' ;
              unset $string;
              set $string htmlid;
              trigger clean_string;
              put $string;
              put '"';
          done;
          putq ' target=' target;
       finish:
          put '"' /if exists($coord_started);
          unset $coord_started;
          put $empty_tag_suffix;
          put '/>' CR;
   end;

       define event pre_post_text;
            set $string pretext;
            trigger clean_string;
            putq ' prefix=' $string;
            unset $string;
            set $string posttext;
            trigger clean_string;
            putq ' suffix=' $string;
            unset $string;
       end;


        define event verbatim_container;
           start:
             open view;
             trigger text finish;
             trigger bodysection;
             put '<Text';
             trigger stylename;
             trigger text_align;
             trigger style_inline;
             putq " style:padding=" cellpadding;
             put ' style:whiteSpace="pre"';
               /*           trigger text_align;*/
               /*
               put ' style:fontFamily="SAS Monospace, Courier New, Courier, monospace" style:fontSize="small">' nl;
               */
             put ">" nl;
             close;
           finish:
             open view;
             put '</Text>' nl;
             close;
        end;


       /*--------------------------------------------------------eric-*/
       /*-- This is for 913.  The old way of doing batch.           --*/
       /*-----------------------------------------------------16Apr04-*/
        define event preformatted_text;
           start:
             open view;
             put '<Text';
/*           trigger text_align;*/
             trigger stylename;
             trigger text_align;
             trigger style_inline;
             trigger preformatted;
               /*
               put ' style:fontFamily="SAS Monospace, Courier New, Courier, monospace" style:fontSize="small">' nl;
               */
             put ">" nl;
             trigger fit_to_container;
             close;
           finish:
             open view;
             put '</Text>' nl;
             close;
        end;

        define event verbatim;
           start:
             open view;
             trigger fit_to_container;
             close;
        end;

        /*-------------------------------------------------------eric-*/
        /*-- This is event is no longer used.  Even when doing the  --*/
        /*-- batch view, normal pagebreaks are used.   The verbatim --*/
        /*-- section will be ended, and a pagebreak inserted.       --*/
        /*--                                                        --*/
        /*-- Maybe there is something I don't understand here.  But --*/
        /*-- I don't think so.                                      --*/
        /*----------------------------------------------------20Dec03-*/
        define event preformatted_pagebreak_text;
           start:
               open view;
               put '<Text';
/*             trigger text_align;*/
               trigger stylename;
               trigger text_align;
               trigger style_inline;
               /*
               put ' style:pageBreakBefore="always" style:fontFamily="SAS Monospace, Courier New, Courier, monospace" style:fontSize="small">' nl;
               */
               put ">" nl;
               trigger fit_to_container;
               close;
           finish:
             open view;
             put '</Text>' nl;
             close;
        end;

        define event verbatim_text;
           open view;
           put "<Span>";
           put value ;
           put "<Br/>" nl;
           put "</Span>" nl;
           close;
       end;

       /*--------------------------------------------------------eric-*/
       /*-- This is for 913.  The old way of doing batch.           --*/
       /*-----------------------------------------------------16Apr04-*/
        define event preformatted_line;
           open view;
           put "<Span>";
           put value ;
           put "<Br/>" nl;
           put "</Span>" nl;
           close;
        end;
/*
        define event contents_body;
           start:
           trigger section;
           finish:
           trigger section;
        end;
*/

        /* Format-related events */
        define event mappings;
        end;

        define event mapping;
           start:
              open mappings;
              put '<Format';
              putq ' name=' name;
              put '>' nl;
              ndent;
              put '<Type>';
              put type;
              put '</Type>' nl;
              close;             /* DO NOT xdent! */
           finish:
              open mappings;
              xdent;
              put '</Format>' nl;
              close;
        end;

        define event map_head;
        end;

        define event map_options;
           open mappings;
           put '<Name-Options';
           putq ' Default=' defwidth;
           putq ' Fuzz=' fuzz;
           putq ' Max=' max;
           putq ' Min=' min;
           put  ' Multilabel="true"' / if exists(multilabel);
           put  ' Notsorted="true"'  / if exists(notsorted);
           putq ' Round="true"'      / if exists(round);
           put ' />' nl;
           close;
        end;

        define event map_body;
        end;

        define event map_relation;
           start:
              open mappings;

              put '<Mapping>' nl;
              ndent;
              put '<Label-Options';
              putq ' Datatype=' datatype;
              putq ' Fill=' fill;
              putq ' Language=' fmtlang;
              putq ' Multiplier=' multiplier;
              put  ' Noedit="true"'  / if exists(noedit);
              putq ' Prefix=' prefix;
              put ' />' nl;
              close;
           finish:
              open mappings;
              xdent;
              put '</Mapping>' nl;
              close;
        end;

        define event map_interval;
           open mappings;
           put '<Range>';
           put start;

           /* For single-value intervals, the closure */
           /* and end variables will not exist.       */

           put '&lt;-'     / if cmp(closure,"openclosed");
           put '&lt;-&lt;' / if cmp(closure,"openopen");
           put '-&lt;'     / if cmp(closure,"closedopen");
           put '-'         / if cmp(closure,"closedclosed");
           put end;
           put '</Range>' nl;
           close;
        end;

        define event map_display_value;
           open mappings;
           put '<Label>';
           set $string value;
           /*trigger clean_string;*/
           put  $string;
           unset $string;
           put '</Label>' nl;
           close;
        end;
        /* end format-related events */

      define event table_rolap;
         start:
           unset $column_types;
           put '<IQData type="2DEmbedded"' ;
           putq ' name=' name ;
           trigger putall;
           put '>' nl;
           ndent;
           put "<EmbeddedData>" nl;
           ndent;
           trigger view_table_rolap;
         finish:
           do /if $do_csv;
               /* put ']]>' nl; */
               put '</CSV>' nl;
               unset $in_csv ;
           else;
               put '</ValuesList>' nl;
               unset $in_valueslist ;
           done;
           trigger view_table_rolap;
           xdent;
           put "</EmbeddedData>" nl;
           xdent;
           put "</IQData>" nl;
      end;

      define event view_table_rolap;
         start:
           open view;
           trigger text finish;
           trigger page_headers finish;
           trigger proctitleheaders finish;
           trigger bodysection;
           trigger pre_post_image start;
           trigger pre_post_textheader start;
           put "<Table";
           putq ' data=' name;
           trigger output_name;
           trigger putall;
           trigger stylename;
           trigger page_break_before;
           trigger style_inline;
           do /if cellspacing;
               set $size_unit cellspacing;
               trigger parse_size_unit;
               put ' cellSpacing="' $size_unit_size $size_unit_unit '"';
               unset $size_unit_size;
               unset $size_unit_unit;
           done;
           do /if cellpadding;
               set $size_unit cellpadding;
               trigger parse_size_unit;
               put ' cellPadding="' $size_unit_size $size_unit_unit '"';
               unset $size_unit_size;
               unset $size_unit_unit;
           done;
           putq  ' style:borderStyle=' $borderStyles[frame];
           put '>' nl;
           trigger griddedconstraint;
           ndent;
           close;
         finish:
           open view;
           xdent;
           put "</Table>" nl;
           trigger pre_post_textheader finish;
           trigger pre_post_image finish;
           close;
      end;

        define event view_columns_rolap;
          start:
            open view;
            put '<Columns>' nl;
            ndent;
            close;
          finish:
            open view;
            xdent;
            put '</Columns>' nl;
            close;
        end;

        define event cell_rolap;
            open view;
            put '<Cell style:style="Data" style:textAlign="right" style:verticalAlign="bottom" />' nl;
            close;
        end;

        define event view_column_rolap;
          start:
            open view;
            put '<Column';
            putq ' variable=' name;
            put '>' nl ;
            ndent;
            trigger cell_rolap / if name;
            close;
          finish:
            open view;
            xdent;
            put '</Column>' nl;
            close;
        end;

        define event view_rows_rolap;
          start:
            open view;
            put '<Rows>' nl;
            ndent;
            close;
          finish:
            open view;
            xdent;
            put '</Rows>' nl;
            close;
        end;

        define event view_row_rolap;
          start:
            open view;
            put '<Row>' nl;
            ndent;
            trigger cell_rolap / if name;
            close;
          finish:
            open view;
            xdent;
            put '</Row>' nl;
            close;
        end;

        define event variables_rolap;
          start:
            put '<Variables>' nl;
            ndent;
          finish:
            xdent;
            put '</Variables>' nl;
            close;
        end;

        define event table_body_rolap;
          start:
            trigger table_body;
          finish:
            trigger table_body;
        end;

      define event label_rolap;
        /* It is critically important that the <Label>text</Label> */
        /* sequence not have any extra white space between the     */
        /* tags and the text!                                      */
        open view;
        put  '<Label';
        trigger event_attr;
        put  ' isVisible="false"' / if !exists(value);
        putq ' position=' position;
        do/if !cmp(ROWSPAN, '0');
            putq " rowSpan=" ROWSPAN /if !cmp(ROWSPAN, '1');
        done;
        do/if !cmp(COLSPAN, '0');
            putq " colSpan=" COLSPAN /if !cmp(COLSPAN, '1');
        done;
        trigger stylename;
        trigger text_align;
        trigger vert_text_align;
        trigger style_inline;
        trigger preformatted;
        do /if url;
            eval $link_count $link_count + 1;
            put ' action="link' $link_count '"';
        done;
        put '>';
          /*
        set $string value;
        trigger clean_string;
          put  $string;
          */
        trigger pre_post_text_value start;
        put value;
        trigger pre_post_text_value finish;
        put '</Label>' nl;
        close;
        trigger do_link;
      end;

      define event variable_rolap;
          put  '<Variable';
          /*-------------------------------------------------------eric-*/
          /*-- change these 1st three back to numeric when BIP is     --*/
          /*-- ready to use them as numerics...                       --*/
          /*-- Apparently formatted will forever be strings...        --*/
          /*-- Changing back to String.    08/10/05                   --*/
          /*----------------------------------------------------5Feb 03-*/

          do /if $do_csv;

              do /if $do_formatted;
                  put  ' encoding="text"';
                  do /if cmp(type, "double") | cmp(type, "int") | cmp(type, "bool") ;
                     put ' type="String"';
                  else;
                     put ' type="String"';
                  done;
                  set $column_types[] type;

              else;
                  trigger print_unformat;
                  set $column_types[] unformattedtype;

                  do /if cmp(unformattedtype, "double");
                      put ' encoding="Base64" type="Numeric"';
                  else /if cmp(unformattedtype, "int") | cmp(unformattedtype, "bool") ;
                      put ' encoding="text" type="Numeric"';
                  else;
                      put ' encoding="text" type="String"';
                  done;
              done;
          else;
              put  ' encoding="Text"';
              put  ' type=';
              put  '"String"' / if cmp(type, "double");
              put  '"String"' / if cmp(type, "int");
              put  '"String"' / if cmp(type, "bool");
              put  '"String"'  / if cmp(type, "char");
              put  '"String"'  / if cmp(type, "string");
              put  '"String"'  / if cmp(type, "unknown");
          done;
          set $string name;
          trigger clean_string;
          putq ' name=' strip($string);
          unset $string;
          put  ' />' nl;
      end;

      define event caption_rolap;
          start:
              open view;
              put '<Caption';
              put  ' isVisible="false"' / if !exists(value);
              put  ' isStacked="true"'  / if is_stacked;
              put  ' isStacked="false"' / if !is_stacked;
              trigger stylename;
              trigger style_inline;
              trigger text_align;
              trigger vert_text_align;
              put  '>';
              set $string value;
              do /if contains($string, '&amp' ); /* S0713950 */
                 trigger partial_clean_string;
              else;
                 trigger clean_string;
              done;
              put  $string;
              unset $string;
          finish:
              open view;  /* sometimes it gets closed. */
              put "</Caption>" nl;
              close;
      end;

      define event table_label_rolap;
          start:
              open view;
              put '<Label';
              put  ' isVisible="false"' / if !exists(value);
              put  ' isStacked="true"'  / if is_stacked;
              put  ' isStacked="false"' / if !is_stacked;
              do / if position;
                  putq ' position=' position;
              else;
                  putq ' position="bottom"';
              done;
              trigger stylename;
              trigger style_inline;
              trigger text_align;
              trigger vert_text_align;
              trigger preformatted;
              put  '>';
              trigger pre_post_text_value start;
              set $string value;
              /*trigger clean_string;*/
              put  $string;
              unset $string;
              trigger pre_post_text_value finish;
          finish:
              open view;  /* sometimes it gets closed. */
              put "</Label>" nl;
              close;
      end;

      define event layout_gutters;
        do/if $size_unit;
          trigger parse_size_unit;
          put ' rowGutter="' $size_unit_size $size_unit_unit '"';
          put ' columnGutter="' $size_unit_size $size_unit_unit '"';
          unset $size_unit_size;
          unset $size_unit_unit;
        done;
      end; /* layout_gutters */

      define event layout;
        start:
        /*-----------------------------------------------------sasdck-*/
        /*-- The outermost layout is Sectioned, others are          --*/
        /*-- Containered.  You can't nest GriddedLayouts directly,  --*/
        /*-- but you can nest them indirectly using Containers.     --*/
        /*--                                                        --*/
        /*-- We suppress page breaks in layouts; they're extraneous --*/
        /*-- and make the presentation look too busy.  We also      --*/
        /*-- suppress Sections, because Sections have infinite      --*/
        /*-- width, and thus cannot be nested in Containers, which  --*/
        /*-- may be side-by-side in a layout.                       --*/
        /*--                                                        --*/
        /*-- Headers and Footers also have infinite width, so we    --*/
        /*-- can't use them within a layout for the same reason.    --*/
        /*-- We represent header and footer matter as Containers    --*/
        /*-- when laying them out; see the page_header and footer   --*/
        /*-- events.                                                --*/
        /*----------------------------------------------------26Oct09-*/

          open view;
          do/if $layout_nesting_level eq 0;
            trigger text finish;
            trigger page_headers finish;
            block pagebreak;
            block pagebreak_section;
            block bodysection;
            put '<Section style:pageBreakAfter="always"';
            putq ' style:style=' htmlclass;
            putl '>';
            put '<Body';
            trigger style_inline;
            putl '>';
          else;
            put '<Container';
            putq ' style:style=' htmlclass;
            trigger style_inline;
            putl '>';
          done;
          close;
          set $saw_layout_start "True";
          /*---------------------------------------------------sasdck-*/
          /*-- Currently cellspacing determines both row and column --*/
          /*-- guttering.  This is bogus.  Production 9.3 will      --*/
          /*-- define bona fide row gutter and column gutter style  --*/
          /*-- attributes for the LayoutContainer style.            --*/
          /*--                                                      --*/
          /*-- Other spacings are hardcoded at 10px, which is the   --*/
          /*-- traditional behavior for this tagset.                --*/
          /*--------------------------------------------------26Oct09-*/

          set $layout_cellspacing cellspacing;
          eval $layout_nesting_level $layout_nesting_level + 1;
        finish:
          eval $layout_nesting_level $layout_nesting_level - 1;
          open view;
          do/if $layout_nesting_level eq 0;
            putl '</Body>';
            putl '</Section>';
            unblock pagebreak;
            unblock pagebreak_section;
            unblock bodysection;
          else;
            putl '</Container>';
          done;
          close;
      end; /* layout */

      define event layout_row;
      end; /* layout_row */

      define event layout_region;
        start:
          open view;
          /*-- we delay the layout start until we know the column count --*/
          do/if $saw_layout_start;
            unset $saw_layout_start;
            put '<GriddedLayout';
            putq ' columns=' colcount;
            set $size_unit $layout_cellspacing;
            unset $layout_cellspacing;
            trigger layout_gutters;
            putl '/>';
          done;
          put '<Container';
          set $in_container "TRUE";
          putq ' style:style=' htmlclass;
          putl '>';
          putl '<GriddedLayout rowGutter="10px"/>';
          put '<GriddedConstraint';
          putq ' row=' row;
          putq ' column=' colstart;
          do/if !cmp(rowspan, '0');
            putq " rowSpan=" rowspan /if !cmp(rowspan, '1');
          done;
          do/if !cmp(colspan, '0');
            putq " columnSpan=" colspan /if !cmp(colspan, '1');
          done;
          put ' halign=';
          put '"center"' / if cmp(just, 'c');
          put '"left"' / if cmp(just, 'l');
          put '"right"' / if cmp(just, 'r');
          put ' fill="horizontal"';
          trigger style_inline;
          putl '/>';
          close;
        finish:
          open view;
          putl '</Container>';
          unset $in_container;
          close;
      end; /* layout_region */

    end; /* sasreport13 */

  %end;
  %mend;
  %tagsets_sasreport;



run;
